@@ -1,5 +1,5 @@
 /* GDA common library
- * Copyright (C) 1998 - 2005 The GNOME Foundation.
+ * Copyright (C) 1998 - 2006 The GNOME Foundation.
  *
  * AUTHORS:
  *	Rodrigo Moya <rodrigo@gnome-db.org>
@@ -29,7 +29,8 @@
 #include <gmodule.h>
 #include <libgda/gda-config.h>
 #include <libgda/gda-data-model-array.h>
-#include <libgda/gda-intl.h>
+#include <libgda/gda-parameter-list.h>
+#include <glib/gi18n-lib.h>
 #include <libgda/gda-log.h>
 #include <libxml/xmlmemory.h>
 #include <libxml/parser.h>
@@ -42,6 +43,7 @@
 #include <io.h>
 #endif
 
+#define GDA_CONFIG_SECTION_DATASOURCES "/apps/libgda/Datasources"
 #define LIBGDA_USER_CONFIG_DIR G_DIR_SEPARATOR_S ".libgda"
 #define LIBGDA_USER_CONFIG_FILE LIBGDA_USER_CONFIG_DIR G_DIR_SEPARATOR_S "config"
 
@@ -89,7 +91,7 @@ static void           fam_unlock_notify ();
 /*
  * Private functions
  */
-#ifdef GDA_DEBUG
+#ifdef GDA_DEBUG_NO
 static void
 dump_config_client ()
 {
@@ -432,7 +434,7 @@ get_config_client ()
 				g_warning ("Config file is not readable.");
 		}
 		g_free (user_config);
-#ifdef GDA_DEBUG
+#ifdef GDA_DEBUG_NO
 		dump_config_client ();
 #endif
 	}
@@ -451,7 +453,6 @@ fam_callback (GIOChannel *source, GIOCondition condition, gpointer data)
 	while (fam_connection && FAMPending (fam_connection)) {
                 FAMEvent ev;
 		gboolean is_global;
-		gchar *msg = NULL;
 		
 		if (FAMNextEvent (fam_connection, &ev) != 1) {
                         FAMClose (fam_connection);
@@ -470,7 +471,7 @@ fam_callback (GIOChannel *source, GIOCondition condition, gpointer data)
 		case FAMChanged:
 		case FAMDeleted:
 		case FAMCreated:
-#ifdef GDA_DEBUG
+#ifdef GDA_DEBUG_NO
 			g_print ("Reloading config files (%s config has changed)\n", is_global ? "global" : "user");
 #endif
 			gda_config_client_reset ();
@@ -554,8 +555,10 @@ gda_config_search_entry (GList *sections, const gchar *path, const gchar *type)
 			    !strcmp (entry->type, type) && 
 			    !strcmp (entry->name, ptr_last_dash + 1))
 				break;
-			else if (!strcmp (entry->name, ptr_last_dash + 1))
-				break;
+			else {
+				if (!strcmp (entry->name, ptr_last_dash + 1))
+					break;
+			}
 
 			entry = NULL;
 		}
@@ -774,11 +777,10 @@ gda_config_get_string (const gchar *path)
 	cfg_client = get_config_client ();
 
 	entry = gda_config_search_entry (cfg_client->user, path, "string");
-	if (entry == NULL)
+	if (!entry)
 		entry = gda_config_search_entry (cfg_client->global, path, "string");
 
-        return (entry != NULL && entry->value != NULL) ? 
-			g_strdup (entry->value) : NULL;
+        return (entry && entry->value) ? g_strdup (entry->value) : NULL;
 }
 
 /**
@@ -866,7 +868,7 @@ gda_config_get_boolean (const gchar *path)
  *
  * Sets the given configuration entry to contain a string.
  *
- * Returns: TRUE if no error occured
+ * Returns: TRUE if no error occurred
  */
 gboolean
 gda_config_set_string (const gchar *path, const gchar *new_value)
@@ -920,7 +922,7 @@ gda_config_set_string (const gchar *path, const gchar *new_value)
  *
  * Sets the given configuration entry to contain an integer.
  *
- * Returns: TRUE if no error occured
+ * Returns: TRUE if no error occurred
  */
 gboolean
 gda_config_set_int (const gchar *path, gint new_value)
@@ -975,7 +977,7 @@ gda_config_set_int (const gchar *path, gint new_value)
  *
  * Sets the given configuration entry to contain a float.
  *
- * Returns: TRUE if no error occured
+ * Returns: TRUE if no error occurred
  */
 gboolean
 gda_config_set_float (const gchar * path, gdouble new_value)
@@ -1030,7 +1032,7 @@ gda_config_set_float (const gchar * path, gdouble new_value)
  *
  * Sets the given configuration entry to contain a boolean.
  *
- * Returns: TRUE if no error occured
+ * Returns: TRUE if no error occurred
  */
 gboolean
 gda_config_set_boolean (const gchar *path, gboolean new_value)
@@ -1265,18 +1267,20 @@ gda_config_list_sections_raw (const gchar *path)
 	cfg_client = get_config_client ();
 	for (list = cfg_client->user; list; list = list->next){
 		section = (GdaConfigSection*) list->data;
-		if (section && len < strlen (section->path) && 
-		    !strncmp (path, section->path, len))
+		if (section && 
+		    (len < strlen (section->path)) && 
+		    !strncmp (path, section->path, len) && 
+		    ((section->path[len] == 0) || (section->path[len] == '/')))
 			ret = g_list_append (ret, section);
 	}
 		
 	for (list = cfg_client->global; list; list = list->next){
 		section = (GdaConfigSection*) list->data;
-		if (section && len < strlen (section->path) && 
-		    !strncmp (path, section->path, len)){
-			if (!g_list_find_custom (ret, 
-						 section->path + len + 1,
-						 (GCompareFunc) strcmp))
+		if (section && 
+		    (len < strlen (section->path)) && 
+		    !strncmp (path, section->path, len) &&
+		    ((section->path[len] == 0) || (section->path[len] == '/'))){
+			if (!g_list_find_custom (ret, section->path + len + 1, (GCompareFunc) strcmp))
 				ret = g_list_append (ret, section);
 		}
 	}
@@ -1418,7 +1422,6 @@ gda_config_get_provider_list (void)
 			gchar *ext;
 			const gchar * (* plugin_get_name) (void);
 			const gchar * (* plugin_get_description) (void);
-			GList * (* plugin_get_cnc_params) (void);
 			gchar * (* plugin_get_dsn_spec) (void);
 			
 			ext = g_strrstr (name, ".");
@@ -1440,8 +1443,6 @@ gda_config_get_provider_list (void)
 					 (gpointer *) &plugin_get_name);
 			g_module_symbol (handle, "plugin_get_description",
 					 (gpointer *) &plugin_get_description);
-			g_module_symbol (handle, "plugin_get_connection_params",
-					 (gpointer *) &plugin_get_cnc_params);
 			g_module_symbol (handle, "plugin_get_dsn_spec",
 					 (gpointer *) &plugin_get_dsn_spec);
 			
@@ -1458,17 +1459,35 @@ gda_config_get_provider_list (void)
 			else
 				info->description = NULL;
 			
-			if (plugin_get_cnc_params != NULL)
-				info->gda_params = plugin_get_cnc_params ();
-			else
-				info->gda_params = NULL;
-			
-			if (plugin_get_dsn_spec != NULL)
+			info->dsn_spec = NULL;
+			info->gda_params = NULL;
+
+			if (plugin_get_dsn_spec) {
+				GError *error = NULL;
+				
 				info->dsn_spec = plugin_get_dsn_spec ();
-			else
-				info->dsn_spec = NULL;
+				if (info->dsn_spec) {
+					info->gda_params = gda_parameter_list_new_from_spec_string (NULL, info->dsn_spec, &error);
+					if (!info->gda_params) {
+						g_warning ("Invalid format for provider '%s' DSN spec : %s",
+							   info->id,
+							   error ? error->message : "Unknown error");
+						if (error)
+							g_error_free (error);
+					}
+				}
+				else {
+					/* there may be traces of the provider installed but some parts are missing,
+					 forget about that provider... */
+					gda_provider_info_free (info);
+					info = NULL;
+				}
+			}
+			else 
+				g_warning ("Provider '%s' does not provide a DSN spec", info->id);
 			
-			list = g_list_append (list, info);
+			if (info)
+				list = g_list_append (list, info);
 			
 			g_module_close (handle);
 		}
@@ -1517,7 +1536,7 @@ gda_config_get_provider_by_name (const gchar *name)
 {
 	GList *prov_list;
 	GList *l;
-	gchar *tmpname;
+	const gchar *tmpname;
 
 	if (name)
 		tmpname = name;
@@ -1564,86 +1583,30 @@ gda_config_get_provider_model (void)
 	for (l = prov_list; l != NULL; l = l->next) {
 		GList *value_list = NULL;
 		GdaProviderInfo *prov_info = (GdaProviderInfo *) l->data;
+		GValue *value;
 
 		g_assert (prov_info != NULL);
 
-		value_list = g_list_append (value_list, gda_value_new_string (prov_info->id));
-		value_list = g_list_append (value_list, gda_value_new_string (prov_info->location));
-		value_list = g_list_append (value_list, gda_value_new_string (prov_info->description));
+		value = g_value_init (g_new0 (GValue, 1), G_TYPE_STRING);
+		g_value_set_string (value, prov_info->id);
+		value_list = g_list_append (value_list, value);
 
-		gda_data_model_append_values (GDA_DATA_MODEL (model), value_list);
-		/* FIXME: correct memory leak here by freeing value_list */
+		value = g_value_init (g_new0 (GValue, 1), G_TYPE_STRING);
+		g_value_set_string (value, prov_info->location);
+		value_list = g_list_append (value_list, value);
+
+		value = g_value_init (g_new0 (GValue, 1), G_TYPE_STRING);
+		g_value_set_string (value, prov_info->description);
+		value_list = g_list_append (value_list, value);
+
+		gda_data_model_append_values (GDA_DATA_MODEL (model), value_list, NULL);
+		g_list_foreach (value_list, (GFunc) gda_value_free, NULL);
+		g_list_free (value_list);
 	}
 	
 	return model;
 }
 
-/**
- * gda_provider_parameter_info_new_full:
- * @name:
- * @short_description:
- * @long_description:
- * @type:
- */
-GdaProviderParameterInfo *
-gda_provider_parameter_info_new_full (const gchar *name,
-				      const gchar *short_description,
-				      const gchar *long_description,
-				      GdaValueType type)
-{
-	GdaProviderParameterInfo *param_info;
-
-	param_info = g_new0 (GdaProviderParameterInfo, 1);
-	param_info->name = g_strdup (name);
-	param_info->short_description = g_strdup (short_description);
-	param_info->long_description = g_strdup (long_description);
-	param_info->type = type;
-
-	return param_info;
-}
-
-/**
- * gda_provider_parameter_info_copy:
- * @param_info: a #GdaProviderParameterInfo structure.
- *
- * Creates a new #GdaProviderParameterInfo structure with the values
- * contained in the given @param_info argument..
- *
- * Returns: a newly allocated #GdaProviderParameterInfo structure.
- */
-GdaProviderParameterInfo *
-gda_provider_parameter_info_copy (GdaProviderParameterInfo *param_info)
-{
-	GdaProviderParameterInfo *new_info;
-
-	g_return_val_if_fail (param_info != NULL, NULL);
-
-	new_info = g_new0 (GdaProviderParameterInfo, 1);
-	new_info->name = g_strdup (param_info->name);
-	new_info->short_description = g_strdup (param_info->short_description);
-	new_info->long_description = g_strdup (param_info->long_description);
-	new_info->type = param_info->type;
-
-	return new_info;
-}
-
-/**
- * gda_provider_parameter_info_free:
- * @param_info: a #GdaProviderParameterInfo structure to be freed.
- *
- * Free the memory associated with the given @param_info argument.
- */
-void
-gda_provider_parameter_info_free (GdaProviderParameterInfo *param_info)
-{
-	g_return_if_fail (param_info != NULL);
-
-	g_free (param_info->name);
-	g_free (param_info->short_description);
-	g_free (param_info->long_description);
-	g_free (param_info);
-}
-
 /**
  * gda_provider_info_copy
  * @src: provider information to get a copy from.
@@ -1657,8 +1620,6 @@ GdaProviderInfo*
 gda_provider_info_copy (GdaProviderInfo *src)
 {
 	GdaProviderInfo *info;
-	GList *list = NULL;
-	GList *list_src = NULL;
 
 	g_return_val_if_fail (src != NULL, NULL);
 
@@ -1666,16 +1627,11 @@ gda_provider_info_copy (GdaProviderInfo *src)
 	info->id = g_strdup (src->id);
 	info->location = g_strdup (src->location);
 	info->description = g_strdup (src->description);
-
-	/* Deep copy: */
-	list_src = src->gda_params;
-	while (list_src) {
-		list = g_list_append (list, gda_provider_parameter_info_copy (list_src->data));
-		list_src = g_list_next (list_src);
+	if (src->gda_params) {
+		info->gda_params = src->gda_params;
+		g_object_ref (info->gda_params);
 	}
 
-	info->gda_params = list;
-
 	return info;
 }
 
@@ -1693,8 +1649,8 @@ gda_provider_info_free (GdaProviderInfo *provider_info)
 	g_free (provider_info->id);
 	g_free (provider_info->location);
 	g_free (provider_info->description);
-	g_list_foreach (provider_info->gda_params, (GFunc) gda_provider_parameter_info_free, NULL);
-	g_list_free (provider_info->gda_params);
+	if (provider_info->gda_params)
+		g_object_unref (provider_info->gda_params);
 	if (provider_info->dsn_spec)
 		g_free (provider_info->dsn_spec);
 
@@ -1934,18 +1890,41 @@ gda_config_get_data_source_model (void)
 	for (l = dsn_list; l != NULL; l = l->next) {
 		GList *value_list = NULL;
 		GdaDataSourceInfo *dsn_info = (GdaDataSourceInfo *) l->data;
+		GValue *value;
 
 		g_assert (dsn_info != NULL);
 
-		value_list = g_list_append (value_list, gda_value_new_string (dsn_info->name));
-		value_list = g_list_append (value_list, gda_value_new_string (dsn_info->provider));
-		value_list = g_list_append (value_list, gda_value_new_string (dsn_info->cnc_string));
-		value_list = g_list_append (value_list, gda_value_new_string (dsn_info->description));
-		value_list = g_list_append (value_list, gda_value_new_string (dsn_info->username));
-		value_list = g_list_append (value_list, gda_value_new_string ("******"));
-		value_list = g_list_append (value_list, gda_value_new_boolean (dsn_info->is_global));
+		value = g_value_init (g_new0 (GValue, 1), G_TYPE_STRING);
+		g_value_set_string (value, dsn_info->name);
+		value_list = g_list_append (value_list, value);
 
-		gda_data_model_append_values (GDA_DATA_MODEL (model), value_list);
+		value = g_value_init (g_new0 (GValue, 1), G_TYPE_STRING);
+		g_value_set_string (value, dsn_info->provider);
+		value_list = g_list_append (value_list, value);
+
+		value = g_value_init (g_new0 (GValue, 1), G_TYPE_STRING);
+		g_value_set_string (value, dsn_info->cnc_string);
+		value_list = g_list_append (value_list, value);
+
+		value = g_value_init (g_new0 (GValue, 1), G_TYPE_STRING);
+		g_value_set_string (value, dsn_info->description);
+		value_list = g_list_append (value_list, value);
+
+		value = g_value_init (g_new0 (GValue, 1), G_TYPE_STRING);
+		g_value_set_string (value, dsn_info->username);
+		value_list = g_list_append (value_list, value);
+
+		value = g_value_init (g_new0 (GValue, 1), G_TYPE_STRING);
+		g_value_set_string (value, "******");
+		value_list = g_list_append (value_list, value);
+
+		value = g_value_init (g_new0 (GValue, 1), G_TYPE_BOOLEAN);
+		g_value_set_boolean (value, dsn_info->is_global);
+		value_list = g_list_append (value_list, value);
+		
+		gda_data_model_append_values (GDA_DATA_MODEL (model), value_list, NULL);
+		g_list_foreach (value_list, (GFunc) gda_value_free, NULL);
+		g_list_free (value_list);
 	}
 	
 	/* free memory */
@@ -1982,7 +1961,7 @@ gda_config_can_modify_global_config (void)
  * Adds a new data source (or update an existing one) to the GDA
  * configuration, based on the parameters given.
  *
- * Returns: TRUE if no error occured
+ * Returns: TRUE if no error occurred
  */
 gboolean
 gda_config_save_data_source (const gchar *name,
@@ -2079,7 +2058,7 @@ gda_config_save_data_source (const gchar *name,
  * #GdaDataSourceInfo structure containing all the information
  * about the data source.
  *
- * Returns: TRUE if no error occured
+ * Returns: TRUE if no error occurred
  */
 gboolean
 gda_config_save_data_source_info (GdaDataSourceInfo *dsn_info)
@@ -2211,19 +2190,6 @@ gda_config_remove_listener (guint id)
 	}
 }
 
-GType
-gda_provider_parameter_info_get_type (void)
-{
-	static GType our_type = 0;
-
-	if (our_type == 0)
-		our_type = g_boxed_type_register_static ("GdaProviderParameterInfo",
-							 (GBoxedCopyFunc) gda_provider_parameter_info_copy,
-							 (GBoxedFreeFunc) gda_provider_parameter_info_free);
-
-	return our_type;
-}
-
 GType
 gda_provider_info_get_type (void)
 {
