@@ -1,5 +1,5 @@
 /* GDA library
- * Copyright (C) 1998 - 2005 The GNOME Foundation.
+ * Copyright (C) 1998 - 2006 The GNOME Foundation.
  *
  * AUTHORS:
  *	Rodrigo Moya <rodrigo@gnome-db.org>
@@ -25,8 +25,12 @@
 #include <glib/garray.h>
 #include <libgda/gda-data-model-array.h>
 #include <glib/gi18n-lib.h>
+#include <libgda/gda-row.h>
+#include <libgda/gda-data-model.h>
+#include <libgda/gda-data-model-extra.h>
+#include <libgda/gda-util.h>
 
-#define PARENT_TYPE GDA_TYPE_DATA_MODEL_BASE
+#define PARENT_TYPE GDA_TYPE_DATA_MODEL_ROW
 
 struct _GdaDataModelArrayPrivate {
 	/* number of columns in each row */
@@ -48,68 +52,78 @@ static GObjectClass *parent_class = NULL;
  */
 
 static gint
-gda_data_model_array_get_n_rows (GdaDataModelBase *model)
+gda_data_model_array_get_n_rows (GdaDataModelRow *model)
 {
 	g_return_val_if_fail (GDA_IS_DATA_MODEL_ARRAY (model), -1);
 	return GDA_DATA_MODEL_ARRAY (model)->priv->rows->len;
 }
 
 static gint
-gda_data_model_array_get_n_columns (GdaDataModelBase *model)
+gda_data_model_array_get_n_columns (GdaDataModelRow *model)
 {
 	g_return_val_if_fail (GDA_IS_DATA_MODEL_ARRAY (model), -1);
 	return GDA_DATA_MODEL_ARRAY (model)->priv->number_of_columns;
 }
 
-static GdaColumn *
-gda_data_model_array_describe_column (GdaDataModelBase *model, gint col)
+static GdaRow *
+gda_data_model_array_get_row (GdaDataModelRow *model, gint row, GError **error)
 {
 	g_return_val_if_fail (GDA_IS_DATA_MODEL_ARRAY (model), NULL);
 
-	return NULL;
-}
-
-static const GdaRow *
-gda_data_model_array_get_row (GdaDataModelBase *model, gint row)
-{
-	g_return_val_if_fail (GDA_IS_DATA_MODEL_ARRAY (model), NULL);
-
-	if (row >= GDA_DATA_MODEL_ARRAY (model)->priv->rows->len)
+	if (row >= GDA_DATA_MODEL_ARRAY (model)->priv->rows->len) {
+		g_set_error (error, 0, 0,
+			     _("Row %d out of range 0 - %d"), row,
+			     GDA_DATA_MODEL_ARRAY (model)->priv->rows->len- 1);
 		return NULL;
+	}
 
-	return (const GdaRow *) g_ptr_array_index (GDA_DATA_MODEL_ARRAY (model)->priv->rows, row);
+	return GDA_ROW (g_ptr_array_index (GDA_DATA_MODEL_ARRAY (model)->priv->rows, row));
 }
 
-static const GdaValue *
-gda_data_model_array_get_value_at (GdaDataModelBase *model, gint col, gint row)
+static const GValue *
+gda_data_model_array_get_value_at (GdaDataModelRow *model, gint col, gint row)
 {
 	GdaRow *fields;
 
 	g_return_val_if_fail (GDA_IS_DATA_MODEL_ARRAY (model), NULL);
 
-	if (row >= GDA_DATA_MODEL_ARRAY (model)->priv->rows->len)
+	if (GDA_DATA_MODEL_ARRAY (model)->priv->rows->len == 0) {
+		g_warning (_("No row in data model"));
 		return NULL;
+	}
+
+	if (row >= GDA_DATA_MODEL_ARRAY (model)->priv->rows->len) {
+		g_warning (_("Row %d out of range 0 - %d"), row, 
+			   GDA_DATA_MODEL_ARRAY (model)->priv->rows->len - 1);
+		return NULL;
+	}
+
+	if (col >= GDA_DATA_MODEL_ARRAY (model)->priv->number_of_columns) {
+		g_warning (_("Column out %d of range 0 - %d"), col, 
+			   GDA_DATA_MODEL_ARRAY (model)->priv->number_of_columns - 1);
+		return NULL;
+	}
 
 	fields = g_ptr_array_index (GDA_DATA_MODEL_ARRAY (model)->priv->rows, row);
 	if (fields != NULL) {
-		GdaValue *field;
+		GValue *field;
 
 		field = gda_row_get_value (fields, col);
-		return (const GdaValue *) field;
+		return (const GValue *) field;
 	}
 
 	return NULL;
 }
 
 static gboolean
-gda_data_model_array_is_updatable (GdaDataModelBase *model)
+gda_data_model_array_is_updatable (GdaDataModelRow *model)
 {
 	g_return_val_if_fail (GDA_IS_DATA_MODEL_ARRAY (model), FALSE);
 	return TRUE;
 }
 
-static const GdaRow *
-gda_data_model_array_append_values (GdaDataModelBase *model, const GList *values)
+static GdaRow *
+gda_data_model_array_append_values (GdaDataModelRow *model, const GList *values, GError **error)
 {
 	gint len;
 	GdaRow *row = NULL;
@@ -118,38 +132,74 @@ gda_data_model_array_append_values (GdaDataModelBase *model, const GList *values
 	g_return_val_if_fail (values != NULL, NULL);
 
 	len = g_list_length ((GList *) values);
-	if (len != GDA_DATA_MODEL_ARRAY (model)->priv->number_of_columns)
+	if (len != GDA_DATA_MODEL_ARRAY (model)->priv->number_of_columns) {
+		g_set_error (error, 0, GDA_DATA_MODEL_VALUES_LIST_ERROR,
+			     _("Wrong number of values in values list"));
 		return NULL;
-
-	row = gda_row_new_from_list (GDA_DATA_MODEL (model), values);
-	if (row) {
-		g_ptr_array_add (GDA_DATA_MODEL_ARRAY (model)->priv->rows, row);
-		gda_row_set_number (row, GDA_DATA_MODEL_ARRAY (model)->priv->rows->len - 1);
-		gda_data_model_changed (GDA_DATA_MODEL (model));
-		gda_data_model_row_inserted (GDA_DATA_MODEL (model), 
-					     GDA_DATA_MODEL_ARRAY (model)->priv->rows->len - 1);
 	}
 
-	return (const GdaRow *) row;
+	row = gda_row_new_from_list (GDA_DATA_MODEL (model), values);
+	if (!GDA_DATA_MODEL_ROW_CLASS (G_OBJECT_GET_CLASS (model))->append_row (model, row, error)) {
+		g_object_unref (row);
+		return NULL;
+	}
+
+	g_object_unref (row);
+	return (GdaRow *) row;
 }
 
 static gboolean
-gda_data_model_array_append_row (GdaDataModelBase *model, GdaRow *row)
+gda_data_model_array_append_row (GdaDataModelRow *model, GdaRow *row, GError **error)
 {
 	g_return_val_if_fail (GDA_IS_DATA_MODEL_ARRAY (model), FALSE);
 	g_return_val_if_fail (row != NULL, FALSE);
 
 	g_ptr_array_add (GDA_DATA_MODEL_ARRAY (model)->priv->rows, (GdaRow *) row);
+	g_object_ref (row);
 	gda_row_set_number ((GdaRow *) row, GDA_DATA_MODEL_ARRAY (model)->priv->rows->len - 1);
-	gda_data_model_changed (GDA_DATA_MODEL (model));
-	gda_data_model_row_inserted (GDA_DATA_MODEL (model), 
-				     GDA_DATA_MODEL_ARRAY (model)->priv->rows->len - 1);
+	gda_data_model_row_inserted ((GdaDataModel *) model, GDA_DATA_MODEL_ARRAY (model)->priv->rows->len - 1);
 
 	return TRUE;
 }
 
 static gboolean
-gda_data_model_array_remove_row (GdaDataModelBase *model, const GdaRow *row)
+gda_data_model_array_update_row (GdaDataModelRow *model, GdaRow *row, GError **error)
+{
+        gint i;
+        GdaDataModelArrayPrivate *priv;
+	gint num;
+
+        g_return_val_if_fail (GDA_IS_DATA_MODEL_ARRAY (model), FALSE);
+        g_return_val_if_fail (row != NULL, FALSE);
+
+	num = gda_row_get_number (row);
+        priv = GDA_DATA_MODEL_ARRAY (model)->priv;
+
+        for (i = 0; i < priv->rows->len; i++) {
+                if (gda_row_get_number (priv->rows->pdata[i]) == num) {
+			GdaRow *tmp;
+
+			if (row == priv->rows->pdata[i]) {
+				gda_data_model_row_updated ((GdaDataModel *) model, i);
+				return TRUE;
+			}
+
+			tmp = gda_row_copy (row);
+			g_object_unref (priv->rows->pdata[i]);
+			priv->rows->pdata[i] = tmp;
+			gda_data_model_row_updated ((GdaDataModel *) model, i);
+
+                        return TRUE;
+                }
+        }
+
+	g_set_error (error, 0, GDA_DATA_MODEL_ROW_NOT_FOUND_ERROR,
+		     _("Row not found in data model"));
+        return FALSE;
+}
+
+static gboolean
+gda_data_model_array_remove_row (GdaDataModelRow *model, GdaRow *row, GError **error)
 {
 	gint i, rownum;
 	
@@ -160,68 +210,19 @@ gda_data_model_array_remove_row (GdaDataModelBase *model, const GdaRow *row)
 		/* renumber following rows */
 		rownum = gda_row_get_number ((GdaRow *) row);
 		for (i = (rownum + 1); i < GDA_DATA_MODEL_ARRAY (model)->priv->rows->len; i++)
-			gda_row_set_number ((GdaRow *) gda_data_model_get_row (GDA_DATA_MODEL (model), i), (i-1));
+			gda_row_set_number ((GdaRow *) gda_data_model_array_get_row (model, i, error), (i-1));
 
 		/* tag the row as being removed */
 		gda_row_set_id ((GdaRow *) row, "R");
 		gda_row_set_number ((GdaRow *) row, -1);
 
-		gda_data_model_changed (GDA_DATA_MODEL (model));
-		gda_data_model_row_removed (GDA_DATA_MODEL (model), rownum);
+		gda_data_model_row_removed ((GdaDataModel *) model, rownum);
+		g_object_unref (row);
 		return TRUE;
 	}
 
-	return FALSE;
-}
-
-static gboolean
-gda_data_model_array_update_row (GdaDataModelBase *model, const GdaRow *row)
-{
-	gint i;
-	GdaDataModelArrayPrivate *priv;
-
-	g_return_val_if_fail (GDA_IS_DATA_MODEL_ARRAY (model), FALSE);
-	g_return_val_if_fail (row != NULL, FALSE);
-
-	priv = GDA_DATA_MODEL_ARRAY (model)->priv;
-
-	for (i = 0; i < priv->rows->len; i++) {
-		if (priv->rows->pdata[i] == row) {
-			gda_row_free (priv->rows->pdata[i]);
-			priv->rows->pdata[i] = gda_row_copy ((GdaRow *) row);
-			gda_data_model_changed (GDA_DATA_MODEL (model));
-			gda_data_model_row_updated (GDA_DATA_MODEL (model), i);
-
-			return TRUE;
-		}
-	}
-
-	return FALSE; /* row not found in this data model */
-}
-
-static gboolean
-gda_data_model_array_append_column (GdaDataModelBase *model, const GdaColumn *attrs)
-{
-	g_return_val_if_fail (GDA_IS_DATA_MODEL_ARRAY (model), FALSE);
-	g_return_val_if_fail (attrs != NULL, FALSE);
-
-	return FALSE;
-}
-
-static gboolean
-gda_data_model_array_update_column (GdaDataModelBase *model, gint col, const GdaColumn *attrs)
-{
-	g_return_val_if_fail (GDA_IS_DATA_MODEL_ARRAY (model), FALSE);
-	g_return_val_if_fail (attrs != NULL, FALSE);
-
-	return FALSE;
-}
-
-static gboolean
-gda_data_model_array_remove_column (GdaDataModelBase *model, gint col)
-{
-	g_return_val_if_fail (GDA_IS_DATA_MODEL_ARRAY (model), FALSE);
-
+	g_set_error (error, 0, GDA_DATA_MODEL_ROW_NOT_FOUND_ERROR,
+		     _("Row not found in data model"));
 	return FALSE;
 }
 
@@ -229,24 +230,20 @@ static void
 gda_data_model_array_class_init (GdaDataModelArrayClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
-	GdaDataModelBaseClass *model_class = GDA_DATA_MODEL_BASE_CLASS (klass);
+	GdaDataModelRowClass *model_class = GDA_DATA_MODEL_ROW_CLASS (klass);
 
 	parent_class = g_type_class_peek_parent (klass);
 
 	object_class->finalize = gda_data_model_array_finalize;
 	model_class->get_n_rows = gda_data_model_array_get_n_rows;
 	model_class->get_n_columns = gda_data_model_array_get_n_columns;
-	model_class->describe_column = gda_data_model_array_describe_column;
 	model_class->get_row = gda_data_model_array_get_row;
 	model_class->get_value_at = gda_data_model_array_get_value_at;
 	model_class->is_updatable = gda_data_model_array_is_updatable;
 	model_class->append_values = gda_data_model_array_append_values;
 	model_class->append_row = gda_data_model_array_append_row;
-	model_class->remove_row = gda_data_model_array_remove_row;
 	model_class->update_row = gda_data_model_array_update_row;
-	model_class->append_column = gda_data_model_array_append_column;
-	model_class->update_column = gda_data_model_array_update_column;
-	model_class->remove_column = gda_data_model_array_remove_column;
+	model_class->remove_row = gda_data_model_array_remove_row;
 }
 
 static void
@@ -268,6 +265,7 @@ gda_data_model_array_finalize (GObject *object)
 	g_return_if_fail (GDA_IS_DATA_MODEL_ARRAY (model));
 
 	/* free memory */
+	gda_data_model_freeze (model);
 	gda_data_model_array_clear (model);
 	g_ptr_array_free (model->priv->rows, TRUE);
 
@@ -295,9 +293,7 @@ gda_data_model_array_get_type (void)
 			0,
 			(GInstanceInitFunc) gda_data_model_array_init
 		};
-		type = g_type_register_static (PARENT_TYPE,
-					       "GdaDataModelArray",
-					       &info, 0);
+		type = g_type_register_static (PARENT_TYPE, "GdaDataModelArray", &info, 0);
 	}
 	return type;
 }
@@ -306,6 +302,9 @@ gda_data_model_array_get_type (void)
  * gda_data_model_array_new
  * @cols: number of columns for rows in this data model.
  *
+ * Creates a new #GdaDataModel object without initializing the column
+ * types. Using gda_data_model_array_new_with_types() is usually better.
+ *
  * Returns: a pointer to the newly created #GdaDataModel.
  */
 GdaDataModel *
@@ -318,89 +317,88 @@ gda_data_model_array_new (gint cols)
 	return model;
 }
 
-typedef struct {
-	gchar *title;
-	gint   position;
-} XmlColumnSpec;
-
 /**
- * gda_data_model_array_new_from_xml_node
- * @node: an XML node representing a &lt;data-array&gt; tag
- * @error: a place to store errors or %NULL
+ * gda_data_model_array_new_with_types
+ * @cols: number of columns for rows in this data model.
+ * @...: types of the columns of the model to create
+ * 
+ * Creates a new #GdaDataModel object with the column types as
+ * specified.
  *
- * Creates a new #GdaDataModel with the data stored in @node
- *
- * Returns: a pointer to the newly created #GdaDataModel, or %NULL if an error occured
+ * Returns: a pointer to the newly created #GdaDataModel.
  */
 GdaDataModel *
-gda_data_model_array_new_from_xml_node (xmlNodePtr node, GError **error)
+gda_data_model_array_new_with_types (gint cols, ...)
 {
 	GdaDataModel *model;
-	xmlNodePtr cur;
-	gint nbfields = 0;
-	GSList *fields = NULL;
-	GSList *list;
+	va_list args;
+	gint i;
 
-	g_return_val_if_fail (node, NULL);
+	model = gda_data_model_array_new (cols);
+	va_start (args, cols);
+	i = 0;
+	while (i < cols) {
+		gint argtype;
 
-	if (strcmp (node->name, "data-array")) {
-		g_set_error (error, 0, 0, _("Node is not <data-array>: '%s'"), node->name);
-		return NULL;
+		argtype = va_arg (args, GType);
+		g_assert (argtype >= 0);
+
+		gda_column_set_gda_type (gda_data_model_describe_column (model, i), 
+					 (GType) argtype);
+		i++;
 	}
+	va_end (args);
+	return model;
+}
 
-	for (cur = node->children; cur; cur=cur->next) {
-		if (!strcmp (cur->name, "field")) {
-			gchar *str;
-			XmlColumnSpec *spec;
+/**
+ * gda_data_model_array_copy_model
+ * @src: a #GdaDataModel to copy data from
+ * @error: a place to store errors, or %NULL
+ *
+ * Makes a copy of @src into a new #GdaDataModelArray object
+ *
+ * Returns: a new data model, or %NULL if an error occurred
+ */
+GdaDataModel *
+gda_data_model_array_copy_model (GdaDataModel *src, GError **error)
+{
+	GdaDataModel *model;
+	gint nbfields, i;
 
-			spec = g_new0 (XmlColumnSpec, 1);
-			spec->title = xmlGetProp(cur, "name");
-			if (!spec->title)
-				spec->title = g_strdup_printf ("field_%d", nbfields);
-			str = xmlGetProp(cur, "position");
-			if (str) {
-				spec->position = atoi (str);
-				g_free (str);
-			}
-			else
-				spec->position = nbfields;
+	g_return_val_if_fail (GDA_IS_DATA_MODEL (src), NULL);
 
-			fields = g_slist_append (fields, spec);
-			nbfields ++;
-			continue;
-		}
-		if (!strcmp (cur->name, "data"))
-			break;
-	}
-
-	if (nbfields == 0) {
-		g_set_error (error, 0, 0, _("No <field> specified in <data-array>"));
-		return NULL;
-	}
-
-	if (! cur) {
-		g_set_error (error, 0, 0, _("No <data> specified in <data-array>"), node->name);
-		return NULL;
-	}
-
-	/* model creation */
+	nbfields = gda_data_model_get_n_columns (src);
 	model = gda_data_model_array_new (nbfields);
-	if (! gda_data_model_add_data_from_xml_node (model, cur)) {
+
+	gda_object_set_name (GDA_OBJECT (model), gda_object_get_name (GDA_OBJECT (src)));
+	gda_object_set_description (GDA_OBJECT (model), gda_object_get_description (GDA_OBJECT (src)));
+	for (i = 0; i < nbfields; i++) {
+		GdaColumn *copycol, *srccol;
+		gchar *colid;
+
+		srccol = gda_data_model_describe_column (src, i);
+		copycol = gda_data_model_describe_column (model, i);
+
+		g_object_get (G_OBJECT (srccol), "id", &colid, NULL);
+		g_object_set (G_OBJECT (copycol), "id", colid, NULL);
+		gda_column_set_title (copycol, gda_column_get_title (srccol));
+		gda_column_set_defined_size (copycol, gda_column_get_defined_size (srccol));
+		gda_column_set_name (copycol, gda_column_get_name (srccol));
+		gda_column_set_caption (copycol, gda_column_get_caption (srccol));
+		gda_column_set_scale (copycol, gda_column_get_scale (srccol));
+		gda_column_set_dbms_type (copycol, gda_column_get_dbms_type (srccol));
+		gda_column_set_gda_type (copycol, gda_column_get_gda_type (srccol));
+		gda_column_set_position (copycol, gda_column_get_position (srccol));
+	}
+
+	if (! gda_data_model_import_from_model (model, src, NULL, error)) {
 		g_object_unref (model);
-		return NULL;
+		model = NULL;
 	}
-	
-	/* setting model's column titles */
-	list = fields;
-	while (list) {
-		gda_data_model_set_column_title (model, ((XmlColumnSpec *)(list->data))->position,
-						 ((XmlColumnSpec *)(list->data))->title);
-		g_free (((XmlColumnSpec *)(list->data))->title);
-		g_free (list->data);
-		list = g_slist_next (list);
-	}
-	g_slist_free (fields);
-	
+	/*else
+	  gda_data_model_dump (model, stdout);*/
+
 	return model;
 }
 
@@ -411,20 +409,23 @@ gda_data_model_array_new_from_xml_node (xmlNodePtr node, GError **error)
  *
  * Sets the number of columns for rows inserted in this model. 
  * @cols must be greated than or equal to 0.
+ *
+ * Also clears @model's contents.
  */
 void
 gda_data_model_array_set_n_columns (GdaDataModelArray *model, gint cols)
 {
 	g_return_if_fail (GDA_IS_DATA_MODEL_ARRAY (model));
+
+	gda_data_model_array_clear (model);
 	model->priv->number_of_columns = cols;
-	/* FIXME: should rebuild the internal array, removing/adding empty cols */
 }
 
 /**
  * gda_data_model_array_clear
  * @model: the model to clear.
  *
- * Frees all the rows inserted in @model.
+ * Frees all the rows in @model.
  */
 void
 gda_data_model_array_clear (GdaDataModelArray *model)
@@ -432,10 +433,8 @@ gda_data_model_array_clear (GdaDataModelArray *model)
 	g_return_if_fail (GDA_IS_DATA_MODEL_ARRAY (model));
 
 	while (model->priv->rows->len > 0) {
-		GdaRow *row = (GdaRow *) g_ptr_array_index (model->priv->rows, 0);
+		GdaRow *row = (GdaRow *) g_ptr_array_index (model->priv->rows, model->priv->rows->len-1);
 
-		if (row != NULL)
-			gda_row_free (row);
-		g_ptr_array_remove_index (model->priv->rows, 0);
+		gda_data_model_array_remove_row (model, row, NULL);
 	}
 }
