@@ -28,6 +28,9 @@
 #include <fcntl.h>
 #include <errno.h>
 #include <string.h>
+#ifdef HAVE_LOCALE_H
+#include <locale.h>
+#endif
 
 #include <glib/gi18n-lib.h>
 #include <libgda/gda-decl.h>
@@ -97,7 +100,7 @@ struct _GdaDataModelImportPrivate {
 			xmlNodePtr        node;
 		} node;
 	} extract;
-	GSList              *cursor_values; /* list of GdaValues for the current row 
+	GSList              *cursor_values; /* list of GValues for the current row 
 					     * (might be shorter than the number of columns in the data model)*/
 
 	/* general data */
@@ -143,7 +146,7 @@ static gint                 gda_data_model_import_get_n_rows      (GdaDataModel
 static gint                 gda_data_model_import_get_n_columns   (GdaDataModel *model);
 static GdaColumn           *gda_data_model_import_describe_column (GdaDataModel *model, gint col);
 static guint                gda_data_model_import_get_access_flags(GdaDataModel *model);
-static const GdaValue      *gda_data_model_import_get_value_at    (GdaDataModel *model, gint col, gint row);
+static const GValue      *gda_data_model_import_get_value_at    (GdaDataModel *model, gint col, gint row);
 static guint                gda_data_model_import_get_attributes_at (GdaDataModel *model, gint col, gint row);
 static GdaDataModelIter    *gda_data_model_impor_create_iter      (GdaDataModel *model);
 static gboolean             gda_data_model_import_iter_next       (GdaDataModel *model, GdaDataModelIter *iter); 
@@ -344,6 +347,8 @@ gda_data_model_import_dispose (GObject *object)
 				model->priv->extract.csv.delimiter = NULL;
 			}
 			break;
+		case FORMAT_XML_NODE:
+			break;
 		default:
 			g_assert_not_reached ();
 			break;
@@ -382,6 +387,7 @@ gda_data_model_import_finalize (GObject *object)
 		if (model->priv->cursor_values) {
 			g_slist_foreach (model->priv->cursor_values, (GFunc) gda_value_free, NULL);
 			g_slist_free (model->priv->cursor_values);
+			model->priv->cursor_values = NULL;
 		}
 
 		g_free (model->priv);
@@ -400,15 +406,15 @@ find_option_as_string (GdaDataModelImport *model, const gchar *pname)
 
 	param = gda_parameter_list_find_param (model->priv->options, pname);
 	if (param) {
-		const GdaValue *value;
+		const GValue *value;
 		
 		value = gda_parameter_get_value (param);
-		if (value && !gda_value_is_null ((GdaValue *) value)) {
-			if (!gda_value_isa ((GdaValue *) value, GDA_VALUE_TYPE_STRING))
+		if (value && !gda_value_is_null ((GValue *) value)) {
+			if (!gda_value_isa ((GValue *) value, G_TYPE_STRING))
 				g_warning (_("The '%s' parameter must hold a "
 					     "string value, ignored."), pname);
 			else
-				return gda_value_get_string ((GdaValue *) value);	
+				return g_value_get_string ((GValue *) value);	
 		}
 	}
 
@@ -421,15 +427,15 @@ find_option_as_boolean (GdaDataModelImport *model, const gchar *pname, gboolean
 	GdaParameter *param;
 	param = gda_parameter_list_find_param (model->priv->options, pname);
 	if (param) {
-		const GdaValue *value;
+		const GValue *value;
 		
 		value = gda_parameter_get_value (param);
-		if (value && !gda_value_is_null ((GdaValue *) value)) {
-			if (!gda_value_isa ((GdaValue *) value, GDA_VALUE_TYPE_BOOLEAN))
+		if (value && !gda_value_is_null ((GValue *) value)) {
+			if (!gda_value_isa ((GValue *) value, G_TYPE_BOOLEAN))
 				g_warning (_("The '%s' parameter must hold a "
 					     "boolean value, ignored."), pname);
 			else
-				return gda_value_get_boolean ((GdaValue *) value);	
+				return g_value_get_boolean ((GValue *) value);	
 		}
 	}
 
@@ -546,10 +552,11 @@ gda_data_model_import_set_property (GObject *object,
 			break;
 			
 	case FORMAT_CSV:
+		model->priv->extract.csv.escape_char = '"';
 		if (model->priv->options) {
 			const gchar *option;
 			option = find_option_as_string (model, "ENCODING");
-			if (option)
+			if (option) 
 				model->priv->extract.csv.encoding = g_strdup (option);
 			option = find_option_as_string (model, "SEPARATOR");
 			if (option)
@@ -639,12 +646,12 @@ gda_data_model_import_new_file   (const gchar *filename, gboolean random_access,
 }
 
 /**
- * gda_data_model_import_new_file
+ * gda_data_model_import_new_mem
  * @data: a string containng the data to import
  *
  * Creates a new #GdaDataModel object which contains the data stored in the @data string. 
  *
- * <b>Important note:</b> the @data string is not copied for memory efficiency reasons and should not
+ * Important note: the @data string is not copied for memory efficiency reasons and should not
  * therefore be altered in any way as long as the returned data model exists.
  *
  * Returns: a pointer to the newly created #GdaDataModel.
@@ -685,10 +692,91 @@ gda_data_model_import_new_xml_node (xmlNodePtr node)
 }
 
 #ifdef GDA_DEBUG
+
+static void
+_dump_row (GdaDataModelIter *iter, gint n_cols, gchar *stroff)
+{
+	gint i;
+	gchar *sep_col  = " | ";
+
+	for (i = 0; i < n_cols; i++) {
+		GdaParameter *param;
+		gchar *str;
+
+		param = gda_data_model_iter_get_param_for_column (iter, i);
+		str = gda_value_stringify ((GValue *) gda_parameter_get_value (param));
+		if (i != 0)
+			g_print ("%s%s", sep_col, str);
+		else
+			g_print ("%s%s", stroff, str);		
+		g_free (str);
+	}
+	g_print ("\n");
+}
+
 static void
 gda_data_model_import_dump (GdaDataModelImport *model, guint offset)
 {
-	TO_IMPLEMENT;
+	gchar *stroff;
+
+	stroff = g_new0 (gchar, offset+1);
+	memset (stroff, ' ', offset);
+
+	if (model->priv) {
+		GdaDataModelIter *iter;
+		gint n_cols, i;
+		gchar *sep_col  = " | ";
+		gchar *sep_row  = "-+-";
+		gchar sep_fill = '-';
+		
+		g_print ("%s" D_COL_H1 "GdaDataModelImport %p (name=%s, id=%s)\n" D_COL_NOR,
+			 stroff, model, gda_object_get_name (GDA_OBJECT (model)), 
+			 gda_object_get_id (GDA_OBJECT (model)));
+		
+		/* columns */
+		n_cols = gda_data_model_get_n_columns (GDA_DATA_MODEL (model));
+		for (i = 0; i < n_cols; i++) {
+			GdaColumn *col = gda_data_model_describe_column (GDA_DATA_MODEL (model), i);
+                        g_print ("%sModel col %d has type %s\n", stroff, i, 
+				 gda_type_to_string (gda_column_get_gda_type (col)));
+		}
+
+		for (i = 0; i < n_cols; i++) {
+			const gchar *str;
+
+			str = (gchar *) gda_data_model_get_column_title (GDA_DATA_MODEL (model), i);
+			if (i != 0)
+				g_print ("%s%s", sep_col, str);
+			else
+				g_print ("%s%s", stroff, str);
+		}
+		g_print ("\n");
+
+		/* data */
+		for (i = 0; i < n_cols; i++) {
+			gint j;
+			if (i != 0)
+				g_print ("%s", sep_row);
+			else
+				g_print ("%s", stroff);
+			for (j = 0; j < 10; j++)
+				g_print ("%c", sep_fill);
+		}
+		g_print ("\n");
+
+		iter = gda_data_model_create_iter (GDA_DATA_MODEL (model));
+		if (gda_data_model_iter_is_valid (iter)) {
+			g_print ("%sSome parts of the model may be missing...\n", stroff);
+			_dump_row (iter, n_cols, stroff);
+		}
+		while (gda_data_model_iter_move_next (iter))
+			_dump_row (iter, n_cols, stroff);
+		g_object_unref (iter);
+        }
+        else
+                g_print ("%s" D_COL_ERR "Using finalized object %p" D_COL_NOR, stroff, model);
+
+	g_free (stroff);
 }
 #endif
 
@@ -762,7 +850,7 @@ init_csv_import (GdaDataModelImport *model)
 				arrvalue ++;
 			}
 
-			gda_column_set_gda_type (column, GDA_VALUE_TYPE_STRING);
+			gda_column_set_gda_type (column, G_TYPE_STRING);
 			if (model->priv->options) {
 				gchar *pname;
 				GdaParameter *param;
@@ -771,17 +859,17 @@ init_csv_import (GdaDataModelImport *model)
 				pname = g_strdup_printf ("GDA_TYPE_%d", col);
 				param = gda_parameter_list_find_param (model->priv->options, pname);
 				if (param) {
-					const GdaValue *value;
+					const GValue *value;
 					
 					value = gda_parameter_get_value (param);
-					if (value && !gda_value_is_null ((GdaValue *) value)) {
-						if (!gda_value_isa ((GdaValue *) value, GDA_VALUE_TYPE_TYPE))
+					if (value && !gda_value_is_null ((GValue *) value)) {
+						if (!gda_value_isa ((GValue *) value, G_TYPE_ULONG))
 							g_warning (_("The '%s' parameter must hold a "
 								     "gda type value, ignored."), pname);
 						else {
-							GdaValueType gtype;
+							GType gtype;
 							
-							gtype = gda_value_get_gdatype ((GdaValue *) value);
+							gtype = g_value_get_ulong ((GValue *) value);
 							gda_column_set_gda_type (column, gtype);
 						}
 					}
@@ -794,9 +882,9 @@ init_csv_import (GdaDataModelImport *model)
 					GdaDictType *dtype;
 
 					gda_column_set_dbms_type (column, dbms_t);
-					dtype = gda_dict_get_data_type_by_name (dict, dbms_t);
+					dtype = gda_dict_get_dict_type_by_name (dict, dbms_t);
 					if (dtype) {
-						GdaValueType gtype;
+						GType gtype;
 						
 						gtype = gda_dict_type_get_gda_type (dtype);
 						gda_column_set_gda_type (column, gtype);
@@ -854,6 +942,10 @@ csv_split_line (GdaDataModelImport *model)
 			g_error_free (error);
 		}
 	}
+	else 
+		line = g_locale_to_utf8 (model->priv->extract.csv.line_start,
+					 model->priv->extract.csv.line_end - model->priv->extract.csv.line_start,
+					 NULL, NULL, NULL);
 	
 	if (!line)
 		line = g_strndup (model->priv->extract.csv.line_start, 
@@ -937,11 +1029,10 @@ csv_split_line (GdaDataModelImport *model)
 			ptr = ch;
 			while (*ptr) {
 				/* convert double escape char to a single one */
-				if ((*ptr == esc) && (*(ptr+1) == esc)) {
+				if ((*ptr == esc) && (*(ptr+1) == esc))
 					g_memmove (ptr, ptr+1, len);
-					len--;
-				}
-					
+
+				len--;
 				ptr++;
 			}
 
@@ -1029,8 +1120,8 @@ csv_compute_row_values (GdaDataModelImport *model)
 
 	arrvalue = arr;
 	while (*arrvalue && columns) {
-		GdaValueType gtype;
-		GdaValue *value;
+		GType gtype;
+		GValue *value;
 
 		gtype = gda_column_get_gda_type ((GdaColumn *) columns->data);
 		value = gda_value_new_from_string (*arrvalue, gtype);
@@ -1112,11 +1203,12 @@ static gint xml_fetch_next_xml_node (xmlTextReaderPtr reader);
 static void xml_fetch_next_row (GdaDataModelImport *model);
 
 typedef struct {
+	xmlChar      *id;
 	xmlChar      *name;
 	xmlChar      *title;
 	xmlChar      *caption;
 	xmlChar      *dbms_type;
-	GdaValueType  gdatype;
+	GType         gdatype;
 	gint          size;
 	gint          scale;
 	gboolean      pkey;
@@ -1136,6 +1228,7 @@ clean_field_specs (GSList *fields)
 	list = fields;
 	while (list) {
 		spec = (XmlColumnSpec*)(list->data);
+		xmlFree (spec->id);
 		xmlFree (spec->name);
 		xmlFree (spec->title);
 		xmlFree (spec->caption);
@@ -1193,7 +1286,7 @@ init_xml_import (GdaDataModelImport *model)
 			/* error */
 			gchar *str;
 
-			str = g_strdup_printf (_("Expected <gda-array> node in XML file, got <%s>"), name);
+			str = g_strdup_printf (_("Expected <gda_array> node in XML file, got <%s>"), name);
 			add_error (model, str);
 			g_free (str);
 			xmlFreeTextReader (reader);
@@ -1211,6 +1304,12 @@ init_xml_import (GdaDataModelImport *model)
 			gda_object_set_name (GDA_OBJECT (model), prop);
 			xmlFree (prop);
 		}
+
+		prop = xmlGetProp (node, "descr");
+		if (prop) {
+			gda_object_set_description (GDA_OBJECT (model), prop);
+			xmlFree (prop);
+		}
 		
 		/* compute fields */
 		ret = xml_fetch_next_xml_node (reader);
@@ -1222,6 +1321,7 @@ init_xml_import (GdaDataModelImport *model)
 			spec = g_new0 (XmlColumnSpec, 1);
 			fields = g_slist_append (fields, spec);
 			
+			spec->id = xmlTextReaderGetAttribute (reader, "id");
 			spec->name = xmlTextReaderGetAttribute (reader, "name");
 			spec->title = xmlTextReaderGetAttribute (reader, "title");
 			if (!spec->title && spec->name)
@@ -1298,6 +1398,7 @@ init_xml_import (GdaDataModelImport *model)
 			spec = (XmlColumnSpec *)(list->data);
 			column = gda_column_new ();
 			model->priv->columns = g_slist_append (model->priv->columns, column);
+			g_object_set (G_OBJECT (column), "id", spec->id, NULL);
 			gda_column_set_title (column, spec->title);
 			gda_column_set_name (column, spec->name);
 			gda_column_set_defined_size (column, spec->size);
@@ -1361,12 +1462,21 @@ static void
 xml_fetch_next_row (GdaDataModelImport *model)
 {
 	xmlTextReaderPtr reader;
-	const xmlChar *name, *xmlstr;
+	const xmlChar *name;
 	gint ret;
 
+	const gchar *lang = NULL;
+
 	GSList *columns = model->priv->columns;
+	GdaColumn *last_column = NULL;
 	GSList *values = NULL;
 
+#ifdef HAVE_LC_MESSAGES
+	lang = setlocale (LC_MESSAGES, NULL);
+#else
+	lang = setlocale (LC_CTYPE, NULL);
+#endif
+
 	if (model->priv->cursor_values) {
 		g_slist_foreach (model->priv->cursor_values, (GFunc) gda_value_free, NULL);
 		g_slist_free (model->priv->cursor_values);
@@ -1394,15 +1504,37 @@ xml_fetch_next_row (GdaDataModelImport *model)
 	ret = xml_fetch_next_xml_node (reader);
 	name = (ret > 0) ? xmlTextReaderConstName (reader) : NULL;
 	while (name && !strcmp (name, "gda_value")) {
+		/* ignore this <gda_value> if the "lang" is there and is not the user locale */
+		xmlChar *this_lang;
+		this_lang = xmlTextReaderGetAttribute (reader, "xml:lang");
+		if (this_lang && strncmp (this_lang, lang, strlen (this_lang))) {
+			xmlFree (this_lang);
+			ret = xml_fetch_next_xml_node (reader);
+			name = (ret > 0) ? xmlTextReaderConstName (reader) : NULL;
+			continue;
+		}
+
+		/* use this <gda_value> */
 		if (!columns) 
 			add_error (model, _("Row has too many values (which are ignored)"));
 		else {
 			ret = xmlTextReaderRead (reader);
 			if (ret > 0) {
-				GdaValue *value;
+				GValue *value;
+				GdaColumn *column;
+
+				if (this_lang)
+					column = last_column;
+				else {
+					column = (GdaColumn *) columns->data;
+					last_column = column;
+					columns = g_slist_next (columns);
+				}
 				if (xmlTextReaderNodeType (reader) == XML_TEXT_NODE) {
-					GdaValueType gtype;
-					gtype = gda_column_get_gda_type ((GdaColumn *) columns->data);
+					GType gtype;
+					const xmlChar *xmlstr;
+
+					gtype = gda_column_get_gda_type (column);
 					xmlstr = xmlTextReaderConstValue (reader);
 					/* g_print ("Convert #%s# to %s\n", (gchar *) xmlstr, gda_type_to_string (gtype)); */
 					value = gda_value_new_from_string ((gchar *) xmlstr, gtype);
@@ -1420,9 +1552,14 @@ xml_fetch_next_row (GdaDataModelImport *model)
 					/* no contents => this is a NULL value */
 					value = gda_value_new_null ();
 
-				values = g_slist_prepend (values, value);
-				
-				columns = g_slist_next (columns);
+				if (this_lang) {
+					/* replace the last value (which did not have any "lang" attribute */
+					gda_value_free ((GValue *) values->data);
+					values->data = value;
+					xmlFree (this_lang);
+				}
+				else 
+					values = g_slist_prepend (values, value);
 			}
 		}
 		ret = xml_fetch_next_xml_node (reader);
@@ -1458,7 +1595,7 @@ init_node_import (GdaDataModelImport *model)
 	if (strcmp (node->name, "gda_array")) {
 		gchar *str;
 
-		str = g_strdup_printf (_("Expected XML node <gda_array> but got <%s>"), node->name);
+		str = g_strdup_printf (_("Expected <gda_array> node but got <%s>"), node->name);
 		add_error (model, str);
 		g_free (str);
 
@@ -1474,7 +1611,8 @@ init_node_import (GdaDataModelImport *model)
 
 			spec = g_new0 (XmlColumnSpec, 1);
 			fields = g_slist_append (fields, spec);
-
+			
+			spec->id = xmlGetProp (cur, "id");
 			spec->name = xmlGetProp (cur, "name");
 			spec->title = xmlGetProp (cur, "title");
 			if (!spec->title && spec->name)
@@ -1534,7 +1672,7 @@ init_node_import (GdaDataModelImport *model)
 			break;
 	}
 
-	if ((nbfields == 0) || !cur) {
+	if (nbfields == 0) {
 		add_error (model, _("No <gda_array_field> specified in <gda_array>"));
 		node = model->priv->extract.node.node = NULL;
 		clean_field_specs (fields);
@@ -1556,6 +1694,12 @@ init_node_import (GdaDataModelImport *model)
 		xmlFree (str);
 	}
 
+	str = xmlGetProp (node, "descr");
+	if (str) {
+		gda_object_set_description (GDA_OBJECT (model), str);
+		xmlFree (str);
+	}
+
 	list = fields;
 	pos = 0;
 	while (list) {
@@ -1564,6 +1708,7 @@ init_node_import (GdaDataModelImport *model)
 
 		spec = (XmlColumnSpec *)(list->data);
 		column = gda_data_model_describe_column (ramodel, pos);
+		g_object_set (G_OBJECT (column), "id", spec->id, NULL);
 		gda_column_set_title (column, spec->title);
 		gda_column_set_name (column, spec->name);
 		gda_column_set_defined_size (column, spec->size);
@@ -1585,7 +1730,7 @@ init_node_import (GdaDataModelImport *model)
 	clean_field_specs (fields);
 	model->priv->columns = g_slist_reverse (model->priv->columns);
 
-	if (! gda_data_model_add_data_from_xml_node (ramodel, cur, &error)) 
+	if (cur && ! gda_data_model_add_data_from_xml_node (ramodel, cur, &error)) 
 		add_error (model, error && error->message ? error->message : _("No detail"));
 }
 
@@ -1607,7 +1752,7 @@ add_error (GdaDataModelImport *model, const gchar *err)
  * Get the list of errors which @model has to report. The returned list is a list of
  * #GError structures, and must not be modified
  *
- * Returns: the list of errors, or %NULL
+ * Returns: the list of errors (which must not be modified), or %NULL
  */
 GSList *
 gda_data_model_import_get_errors (GdaDataModelImport *model)
@@ -1706,7 +1851,7 @@ gda_data_model_import_get_access_flags (GdaDataModel *model)
 	return flags;
 }
 
-static const GdaValue *
+static const GValue *
 gda_data_model_import_get_value_at (GdaDataModel *model, gint col, gint row)
 {
 	GdaDataModelImport *imodel;
@@ -1733,7 +1878,6 @@ gda_data_model_import_get_attributes_at (GdaDataModel *model, gint col, gint row
 	g_return_val_if_fail (imodel->priv, 0);
 	
 	flags = GDA_VALUE_ATTR_NO_MODIF;
-	TO_IMPLEMENT;
 	
 	return flags;
 }
@@ -1835,7 +1979,7 @@ gda_data_model_import_iter_next (GdaDataModel *model, GdaDataModelIter *iter)
 		vlist = imodel->priv->cursor_values;
 		while (plist && vlist) {
 			gda_parameter_set_value (GDA_PARAMETER (plist->data), 
-						 (GdaValue *) vlist->data);
+						 (GValue *) vlist->data);
 			plist = g_slist_next (plist);
 			vlist = g_slist_next (vlist);
 		}
@@ -1908,7 +2052,7 @@ gda_data_model_import_iter_prev (GdaDataModel *model, GdaDataModelIter *iter)
 		vlist = imodel->priv->cursor_values;
 		while (plist && vlist) {
 			gda_parameter_set_value (GDA_PARAMETER (plist->data), 
-						 (GdaValue *) vlist->data);
+						 (GValue *) vlist->data);
 			plist = g_slist_next (plist);
 			vlist = g_slist_next (vlist);
 		}
