@@ -1,6 +1,6 @@
 /* gda-query-field-field.c
  *
- * Copyright (C) 2003 - 2005 Vivien Malerba
+ * Copyright (C) 2003 - 2006 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -29,6 +29,9 @@
 #include "gda-object-ref.h"
 #include "gda-query.h"
 #include "gda-query-target.h"
+#include "gda-dict.h"
+#include "gda-connection.h"
+#include "gda-server-provider.h"
 
 /* 
  * Main static functions 
@@ -93,7 +96,7 @@ enum
 	PROP_QUERY,
 	PROP_VALUE_PROVIDER_OBJECT,
 	PROP_VALUE_PROVIDER_XML_ID,
-	PROP_HANDLER_PLUGIN,
+	PROP_PLUGIN,
 	PROP_TARGET_OBJ,
 	PROP_TARGET_NAME,
 	PROP_TARGET_ID,
@@ -234,8 +237,8 @@ gda_query_field_field_class_init (GdaQueryFieldFieldClass * class)
 	g_object_class_install_property (object_class, PROP_VALUE_PROVIDER_XML_ID,
 					 g_param_spec_string ("value_provider_xml_id", NULL, NULL, NULL,
 							      (G_PARAM_READABLE | G_PARAM_WRITABLE)));
-	g_object_class_install_property (object_class, PROP_HANDLER_PLUGIN,
-                                         g_param_spec_string ("handler_plugin", NULL, NULL, NULL,
+	g_object_class_install_property (object_class, PROP_PLUGIN,
+                                         g_param_spec_string ("entry_plugin", NULL, NULL, NULL,
                                                               (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 
 	g_object_class_install_property (object_class, PROP_TARGET_OBJ,
@@ -472,7 +475,7 @@ gda_query_field_field_set_property (GObject *object,
 				}
 			}
 			break;
-		case PROP_HANDLER_PLUGIN:
+		case PROP_PLUGIN:
 			val =  g_value_get_string (value);
 			if (ffield->priv->plugin) {
 				g_free (ffield->priv->plugin);
@@ -544,8 +547,19 @@ gda_query_field_field_get_property (GObject *object,
 			else
 				g_value_set_string (value, NULL);
 			break;
-		case PROP_HANDLER_PLUGIN:
+		case PROP_PLUGIN:
 			g_value_set_string (value, ffield->priv->plugin);
+			if (! ffield->priv->plugin) {
+				/* try to use the referenced field plugin */
+				GdaObject *obj;
+				obj = gda_object_ref_get_ref_object (ffield->priv->field_ref);
+				if (obj) {
+					gchar *plugin;
+					g_object_get ((GObject*) obj, "entry_plugin", &plugin, NULL);
+					if (plugin) 
+						g_value_take_string (value, plugin);
+				}
+			}
 			break;
 		case PROP_TARGET_OBJ:
 			g_value_set_pointer (value, gda_object_ref_get_ref_object (ffield->priv->target_ref));
@@ -567,7 +581,7 @@ gda_query_field_field_get_property (GObject *object,
 static GObject *
 gda_query_field_field_copy (GdaQueryField *orig)
 {
-	GdaQueryFieldField *qf;
+	GdaQueryFieldField *qf, *newqf;
 	GObject *obj;
 	GdaObject *ref;
 	
@@ -577,31 +591,40 @@ gda_query_field_field_copy (GdaQueryField *orig)
 	obj = g_object_new (GDA_TYPE_QUERY_FIELD_FIELD, 
 			    "dict", gda_object_get_dict (GDA_OBJECT (qf)), 
 			    "query", qf->priv->query, NULL);
+	newqf = GDA_QUERY_FIELD_FIELD (obj);
 	
 	ref = gda_object_ref_get_ref_object (qf->priv->target_ref);
 	if (ref)
-		gda_object_ref_set_ref_object (GDA_QUERY_FIELD_FIELD (obj)->priv->target_ref, ref);
+		gda_object_ref_set_ref_object (newqf->priv->target_ref, ref);
 	else {
 		const gchar *ref_str;
 		GdaObjectRefType ref_type;
 		GType ref_gtype;
 
+		ref_str = gda_object_ref_get_ref_object_name (qf->priv->target_ref);
+		if (ref_str)
+			g_object_set (G_OBJECT (newqf->priv->target_ref), "obj_name", ref_str, NULL);
+
 		ref_str = gda_object_ref_get_ref_name (qf->priv->target_ref, &ref_gtype, &ref_type);
 		if (ref_str)
-			gda_object_ref_set_ref_name (GDA_QUERY_FIELD_FIELD (obj)->priv->target_ref, ref_gtype, ref_type, ref_str);
+			gda_object_ref_set_ref_name (newqf->priv->target_ref, ref_gtype, ref_type, ref_str);
 	}
 
 	ref = gda_object_ref_get_ref_object (qf->priv->field_ref);
 	if (ref)
-		gda_object_ref_set_ref_object (GDA_QUERY_FIELD_FIELD (obj)->priv->field_ref, ref);
+		gda_object_ref_set_ref_object (newqf->priv->field_ref, ref);
 	else {
 		const gchar *ref_str;
 		GdaObjectRefType ref_type;
 		GType ref_gtype;
 
+		ref_str = gda_object_ref_get_ref_object_name (qf->priv->field_ref);
+		if (ref_str)
+			g_object_set (G_OBJECT (newqf->priv->field_ref), "obj_name", ref_str, NULL);
+
 		ref_str = gda_object_ref_get_ref_name (qf->priv->field_ref, &ref_gtype, &ref_type);
 		if (ref_str)
-			gda_object_ref_set_ref_name (GDA_QUERY_FIELD_FIELD (obj)->priv->field_ref, ref_gtype, ref_type, ref_str);
+			gda_object_ref_set_ref_name (newqf->priv->field_ref, ref_gtype, ref_type, ref_str);
 	}
 	
 	if (gda_object_get_name (GDA_OBJECT (orig)))
@@ -620,7 +643,7 @@ gda_query_field_field_copy (GdaQueryField *orig)
 	}
 
 	if (qf->priv->plugin)
-		GDA_QUERY_FIELD_FIELD (obj)->priv->plugin = g_strdup (qf->priv->plugin);
+		newqf->priv->plugin = g_strdup (qf->priv->plugin);
 
 	return obj;
 }
@@ -801,7 +824,7 @@ gda_query_field_field_get_data_type (GdaEntityField *iface)
 	if (gda_query_field_field_activate (GDA_REFERER (iface))) {
 		GdaEntityField *field;
 		field = GDA_ENTITY_FIELD (gda_object_ref_get_ref_object (GDA_QUERY_FIELD_FIELD (iface)->priv->field_ref));
-		return gda_entity_field_get_data_type (field);
+		return gda_entity_field_get_dict_type (field);
 	}
 
 	return NULL;
@@ -859,7 +882,7 @@ gda_query_field_field_save_to_xml (GdaXmlStorage *iface, GError **error)
 		xmlSetProp (node, "object", xmlid);
 	}
 	else {
-		gchar *tmpstr;
+		const gchar *tmpstr;
 
 		tmpstr = gda_object_ref_get_ref_name (field->priv->field_ref, NULL, NULL);
 		if (tmpstr)
@@ -1009,40 +1032,68 @@ gda_query_field_field_load_from_xml (GdaXmlStorage *iface, xmlNodePtr node, GErr
 static gchar *
 gda_query_field_field_render_as_sql (GdaRenderer *iface, GdaParameterList *context, guint options, GError **error)
 {
-	gchar *str = NULL;
 	GdaQueryFieldField *field;
 	GdaObject *fobj;
 	const gchar *tname = NULL, *fname = NULL;
+	gchar *retval = NULL;
+	GdaServerProviderInfo *sinfo = NULL;
+	GdaDict *dict;
+	GdaConnection *cnc;
 
 	g_return_val_if_fail (iface && GDA_IS_QUERY_FIELD_FIELD (iface), NULL);
 	g_return_val_if_fail (GDA_QUERY_FIELD_FIELD (iface)->priv, NULL);
 	field = GDA_QUERY_FIELD_FIELD (iface);
 	g_return_val_if_fail (field->priv, NULL);
 
-	if (! (options & GDA_RENDERER_FIELDS_NO_TARGET_ALIAS)) {
-		GdaObject *tobj;
+	dict = gda_object_get_dict (GDA_OBJECT (iface));
+	cnc = gda_dict_get_connection (dict);
+	if (cnc)
+		sinfo = gda_connection_get_infos (cnc);
 
-		tobj = gda_object_ref_get_ref_object (field->priv->target_ref);
-		if (tobj)
-			tname = gda_query_target_get_alias (GDA_QUERY_TARGET (tobj));
-		else
-			tname = gda_object_ref_get_ref_name (field->priv->target_ref, NULL, NULL);
-	}		
+	if (!sinfo || sinfo->supports_prefixed_fields) {
+		if (! (options & GDA_RENDERER_FIELDS_NO_TARGET_ALIAS)) {
+			GdaObject *tobj;
+			
+			tobj = gda_object_ref_get_ref_object (field->priv->target_ref);
+			if (tobj)
+				tname = gda_query_target_get_alias (GDA_QUERY_TARGET (tobj));
+			else
+				tname = gda_object_ref_get_ref_name (field->priv->target_ref, NULL, NULL);
+		}
+	}
 		
 	fobj = gda_object_ref_get_ref_object (field->priv->field_ref);
 	if (fobj)
 		fname = gda_object_get_name (fobj);
-	else
-		fname = gda_object_ref_get_ref_name (field->priv->field_ref, NULL, NULL);
-	
-	if (tname && fname)
-		return g_strdup_printf ("%s.%s", tname, fname);
 	else {
-		if (fname)
-			return g_strdup (fname);
+		fname = gda_object_ref_get_ref_object_name (field->priv->field_ref);
+		if (!fname)
+			fname = gda_object_ref_get_ref_name (field->priv->field_ref, NULL, NULL);
+	}
+	
+	if (fname) {
+		/* see if we need quotes around returned string */
+		gchar *tmp;
+
+		tmp = g_utf8_strdown (fname, -1);
+		if (((*tmp <= '9') && (*tmp >= '0')) || 
+		    strcmp (tmp, fname)) {
+			g_free (tmp);
+			tmp = (gchar*) fname;
+			fname = g_strdup_printf ("\"%s\"", tmp);
+		}
+		else
+			g_free (tmp);
 	}
 
-	return NULL;
+	if (tname && fname)
+		retval = g_strdup_printf ("%s.%s", tname, fname);
+	else {
+		if (fname)
+			retval = g_strdup (fname);
+	}
+
+	return retval;
 }
 
 static gchar *
@@ -1061,7 +1112,9 @@ gda_query_field_field_render_as_str (GdaRenderer *iface, GdaParameterList *conte
 	if (tobj) {
 		GdaEntity *ent = gda_query_target_get_represented_entity (GDA_QUERY_TARGET (tobj));
 		if (ent)
-			tname = g_strdup_printf ("%s(%s)", gda_query_target_get_alias (GDA_QUERY_TARGET (tobj)));
+			tname = g_strdup_printf ("%s(%s)", 
+						 gda_object_get_name (GDA_OBJECT (ent)), 
+						 gda_query_target_get_alias (GDA_QUERY_TARGET (tobj)));
 		else
 			tname = g_strdup (gda_query_target_get_alias (GDA_QUERY_TARGET (tobj)));
 	}
