@@ -37,90 +37,47 @@
 #include <libgda/gda-util.h>
 #include <libgda/gda-column.h>
 #include <libgda/gda-dict-field.h>
+#ifdef HAVE_LOCALE_H
+#include <locale.h>
+#endif
 
 /**
  * gda_type_to_string
  * @type: Type to convert from.
  *
- * Returns: the string representing the given #GdaValueType.
- * This is not necessarily the same string used to describe the column type in a SQL statement.
- * Use gda_connection_get_schema() with GDA_CONNECTION_SCHEMA_TYPES to get the actual types supported by the provider.
+ * Returns: the string representing the given #GType.
  */
 const gchar *
-gda_type_to_string (GdaValueType type)
+gda_type_to_string (GType type)
 {
-	switch (type) {
-	case GDA_VALUE_TYPE_NULL : return "null";
-	case GDA_VALUE_TYPE_BIGINT : return "bigint";
-	case GDA_VALUE_TYPE_BIGUINT : return "biguint";
-	case GDA_VALUE_TYPE_BLOB : return "blob";
-	case GDA_VALUE_TYPE_BINARY : return "binary";
-	case GDA_VALUE_TYPE_BOOLEAN : return "boolean";
-	case GDA_VALUE_TYPE_DATE : return "date";
-	case GDA_VALUE_TYPE_DOUBLE : return "double";
-	case GDA_VALUE_TYPE_GEOMETRIC_POINT : return "point";
-	case GDA_VALUE_TYPE_INTEGER : return "integer";
-	case GDA_VALUE_TYPE_UINTEGER : return "uinteger";
-	case GDA_VALUE_TYPE_LIST : return "list";
-	case GDA_VALUE_TYPE_NUMERIC : return "numeric";
-	case GDA_VALUE_TYPE_MONEY : return "money"; 
-	case GDA_VALUE_TYPE_SINGLE : return "single";
-	case GDA_VALUE_TYPE_SMALLINT : return "smallint";
-	case GDA_VALUE_TYPE_SMALLUINT : return "smalluint";
-	case GDA_VALUE_TYPE_STRING : return "string";
-	case GDA_VALUE_TYPE_TIME : return "time";
-	case GDA_VALUE_TYPE_TIMESTAMP : return "timestamp";
-	case GDA_VALUE_TYPE_TINYINT : return "tinyint";
-	case GDA_VALUE_TYPE_TINYUINT : return "tinyuint";
-    
-	case GDA_VALUE_TYPE_GOBJECT : return "gobject";
-	case GDA_VALUE_TYPE_TYPE : return "type";
-	case GDA_VALUE_TYPE_UNKNOWN : return "unknown";
-	default: break;
-	}
-
-	return "unknown";
+	if (type == GDA_TYPE_NULL)
+		return "null";
+	else
+		return g_type_name (type);
 }
 
 /**
  * gda_type_from_string
- * @str: the name of a #GdaValueType, as returned by gda_type_to_string().
+ * @str: the name of a #GType, as returned by gda_type_to_string().
  *
- * Returns: the #GdaValueType represented by the given @str.
+ * Returns: the #GType represented by the given @str.
  */
-GdaValueType
+GType
 gda_type_from_string (const gchar *str)
 {
-	g_return_val_if_fail (str != NULL, GDA_VALUE_TYPE_UNKNOWN);
+	GType type;
+	g_return_val_if_fail (str != NULL, G_TYPE_INVALID);
+	
+	type = g_type_from_name (str);
+	if (type == 0) {
+		if (!strcmp (str, "null"))
+			type = GDA_TYPE_NULL;
+		else
+			/* could not find a valid GType for @str */
+			type = G_TYPE_INVALID;
+	}
 
-	if (!g_strcasecmp (str, "null")) return GDA_VALUE_TYPE_NULL;
-	else if (!g_strcasecmp (str, "bigint")) return GDA_VALUE_TYPE_BIGINT;
-	else if (!g_strcasecmp (str, "biguint")) return GDA_VALUE_TYPE_BIGUINT;
-	else if (!g_strcasecmp (str, "binary")) return GDA_VALUE_TYPE_BINARY;
-	else if (!g_strcasecmp (str, "blob")) return GDA_VALUE_TYPE_BLOB;
-	else if (!g_strcasecmp (str, "boolean")) return GDA_VALUE_TYPE_BOOLEAN;
-	else if (!g_strcasecmp (str, "date")) return GDA_VALUE_TYPE_DATE;
-	else if (!g_strcasecmp (str, "double")) return GDA_VALUE_TYPE_DOUBLE;
-	else if (!g_strcasecmp (str, "point")) return GDA_VALUE_TYPE_GEOMETRIC_POINT;
-	else if (!g_strcasecmp (str, "integer")) return GDA_VALUE_TYPE_INTEGER;
-	else if (!g_strcasecmp (str, "uinteger")) return GDA_VALUE_TYPE_UINTEGER;
-	else if (!g_strcasecmp (str, "list")) return GDA_VALUE_TYPE_LIST;
-	else if (!g_strcasecmp (str, "numeric")) return GDA_VALUE_TYPE_NUMERIC;
-	else if (!g_strcasecmp (str, "money")) return GDA_VALUE_TYPE_MONEY;
-	else if (!g_strcasecmp (str, "single")) return GDA_VALUE_TYPE_SINGLE;
-	else if (!g_strcasecmp (str, "smallint")) return GDA_VALUE_TYPE_SMALLINT;
-	else if (!g_strcasecmp (str, "smalluint")) return GDA_VALUE_TYPE_SMALLUINT;  
-	else if (!g_strcasecmp (str, "string")) return GDA_VALUE_TYPE_STRING;
-	else if (!g_strcasecmp (str, "time")) return GDA_VALUE_TYPE_TIME;
-	else if (!g_strcasecmp (str, "timestamp")) return GDA_VALUE_TYPE_TIMESTAMP;
-	else if (!g_strcasecmp (str, "tinyint")) return GDA_VALUE_TYPE_TINYINT;
-	else if (!g_strcasecmp (str, "tinyuint")) return GDA_VALUE_TYPE_TINYUINT;
-  
-	else if (!g_strcasecmp (str, "gobject")) return GDA_VALUE_TYPE_GOBJECT;
-	else if (!g_strcasecmp (str, "type")) return GDA_VALUE_TYPE_TYPE;  
-	else if (!g_strcasecmp (str, "unknown")) return GDA_VALUE_TYPE_UNKNOWN;      
-  
-	return GDA_VALUE_TYPE_UNKNOWN;
+	return type;
 }
 
 /**
@@ -655,7 +612,7 @@ utility_build_decoded_id (const gchar *prefix, const gchar *id)
  * utility_check_data_model
  * @model: a #GdaDataModel object
  * @nbcols: the minimum requested number of columns
- * @Varargs: @nbcols arguments of type GdaValueType or -1 (if any data type is accepted)
+ * @Varargs: @nbcols arguments of type GType or -1 (if any data type is accepted)
  *
  * Check the column types of a GdaDataModel.
  *
@@ -676,7 +633,7 @@ utility_check_data_model (GdaDataModel *model, gint nbcols, ...)
 	/* type of each column */
 	if (nbcols > 0) {
 		GdaColumn *att;
-		GdaValueType mtype, rtype;
+		GType mtype, rtype;
 		gint argtype;
 		va_list ap;
 
@@ -686,9 +643,9 @@ utility_check_data_model (GdaDataModel *model, gint nbcols, ...)
 			att = gda_data_model_describe_column (model, i);
 			mtype = gda_column_get_gda_type (att);
 			
-			argtype = va_arg (ap, GdaValueType);
+			argtype = va_arg (ap, GType);
 			if (argtype >= 0) {
-				rtype = (GdaValueType) argtype;
+				rtype = (GType) argtype;
 				if (mtype != rtype) {
 					retval = FALSE;
 #ifdef GDA_DEBUG
@@ -706,3 +663,300 @@ utility_check_data_model (GdaDataModel *model, gint nbcols, ...)
 	return retval;
 
 }
+
+/**
+ * utility_data_model_dump_data_to_xml
+ * @model: 
+ * @cols: an array containing which columns of @model will be exported, or %NULL for all columns
+ * @nb_cols: the number of columns in @cols
+ * @name: name to use for the XML resulting table.
+ *
+ * Dump the data in a #GdaDataModel into a xmlNodePtr (as used in libxml).
+ */
+void
+utility_data_model_dump_data_to_xml (GdaDataModel *model, xmlNodePtr parent, 
+				     const gint *cols, gint nb_cols, gboolean use_col_ids)
+{
+	gint rows, i;
+	gint *rcols, rnb_cols;
+	gchar **col_ids = NULL;
+
+	/* compute columns if not provided */
+	if (!cols) {
+		rnb_cols = gda_data_model_get_n_columns (model);
+		rcols = g_new (gint, rnb_cols);
+		for (i = 0; i < rnb_cols; i++)
+			rcols [i] = i;
+	}
+	else {
+		rcols = (gint *) cols;
+		rnb_cols = nb_cols;
+	}
+
+	if (use_col_ids) {
+		gint c;
+		col_ids = g_new0 (gchar *, rnb_cols);
+		for (c = 0; c < rnb_cols; c++) {
+			GdaColumn *column;
+			const gchar *id;
+		
+			column = gda_data_model_describe_column (model, rcols [c]);
+			g_object_get (G_OBJECT (column), "id", &id, NULL);
+			if (id && *id)
+				col_ids [c] = g_strdup (id);
+			else
+				col_ids [c] = g_strdup_printf ("_%d", c);
+		}
+	}
+
+	/* add the model data to the XML output */
+	rows = gda_data_model_get_n_rows (model);
+	if (rows > 0) {
+		xmlNodePtr row, data, field;
+		gint r, c;
+
+		data = xmlNewChild (parent, NULL, "gda_array_data", NULL);
+		for (r = 0; r < rows; r++) {
+			row = xmlNewChild (data, NULL, "gda_array_row", NULL);
+			for (c = 0; c < rnb_cols; c++) {
+				GValue *value;
+				gchar *str;
+
+				value = (GValue *) gda_data_model_get_value_at (model, rcols [c], r);
+				if (!value || gda_value_is_null ((GValue *) value))
+					str = NULL;
+				else {
+					if (G_VALUE_TYPE (value) == G_TYPE_BOOLEAN)
+						str = g_strdup (g_value_get_boolean (value) ? "TRUE" : "FALSE");
+					else
+						str = gda_value_stringify (value);
+				}
+				if (!use_col_ids) 
+					field = xmlNewChild (row, NULL, "gda_value", str);
+				else {
+					field = xmlNewChild (row, NULL, "gda_array_value", str);
+					xmlSetProp (field, "colid", col_ids [c]);
+				}
+				if (!str)
+					xmlSetProp (field, "isnull", "t");
+
+				g_free (str);
+			}
+		}
+	}
+
+	if (use_col_ids) {
+		gint c;
+		for (c = 0; c < rnb_cols; c++) 
+			g_free (col_ids [c]);
+		g_free (col_ids);
+	}
+}
+
+/**
+ * utility_parameter_load_attributes
+ * @param:
+ * @node: an xmlNodePtr with a &lt;parameter&gt; tag
+ * @sources: a list of #GdaDataModel
+ *
+ * WARNING: may set the "source" custom string property 
+ */
+void
+utility_parameter_load_attributes (GdaParameter *param, xmlNodePtr node, GSList *sources)
+{
+	xmlChar *str;
+	xmlNodePtr vnode;
+
+	/* set properties from the XML spec */
+	str = xmlGetProp (node, BAD_CAST "id");
+	if (str) {
+		g_object_set (G_OBJECT (param), "string_id", str, NULL);
+		xmlFree (str);
+	}	
+
+	str = xmlGetProp (node, BAD_CAST "name");
+	if (str) {
+		gda_object_set_name (GDA_OBJECT (param), str);
+		xmlFree (str);
+	}
+	str = xmlGetProp (node, BAD_CAST "descr");
+	if (str) {
+		gda_object_set_description (GDA_OBJECT (param), str);
+		xmlFree (str);
+	}
+	str = xmlGetProp (node, BAD_CAST "nullok");
+	if (str) {
+		gda_parameter_set_not_null (param, (*str == 'T') || (*str == 't') ? FALSE : TRUE);
+		xmlFree (str);
+	}
+	else
+		gda_parameter_set_not_null (param, FALSE);
+	str = xmlGetProp (node, BAD_CAST "plugin");
+	if (str) {
+		g_object_set (G_OBJECT (param), "entry_plugin", str, NULL);
+		xmlFree (str);
+	}
+	
+	str = xmlGetProp (node, BAD_CAST "source");
+	if (str) 
+		g_object_set_data_full (G_OBJECT (param), "source", str, g_free);
+
+	/* set restricting source if specified */
+	if (str && sources) {
+		gchar *ptr1, *ptr2 = NULL, *tok;
+		gchar *source;
+			
+		source = g_strdup (str);
+		ptr1 = strtok_r (source, ":", &tok);
+		if (ptr1)
+			ptr2 = strtok_r (NULL, ":", &tok);
+		
+		if (ptr1 && ptr2) {
+			GSList *tmp = sources;
+			GdaDataModel *model = NULL;
+			while (tmp && !model) {
+				if (!strcmp (gda_object_get_name (GDA_OBJECT (tmp->data)), ptr1))
+					model = GDA_DATA_MODEL (tmp->data);
+				tmp = g_slist_next (tmp);
+			}
+			
+			if (model) {
+				gint fno;
+				
+				fno = atoi (ptr2);
+				if ((fno < 0) ||
+				    (fno >= gda_data_model_get_n_columns (model))) 
+					g_warning (_("Field number %d not found in source named '%s'"), fno, ptr1); 
+				else {
+					if (gda_parameter_restrict_values (param, model, fno, NULL)) {
+						/* rename the wrapper with the current param's name */
+						g_object_set_data_full (G_OBJECT (model), "newname", 
+									g_strdup ((gchar *)gda_object_get_name (GDA_OBJECT (param))),
+									g_free);
+						g_object_set_data_full (G_OBJECT (model), "newdescr", 
+									g_strdup ((gchar *)gda_object_get_description (GDA_OBJECT (param))),
+									g_free);
+					}
+				}
+			}
+		}
+	}
+
+	/* specified value */
+	vnode = node->children;
+	if (vnode) {
+		xmlChar *this_lang, *isnull;
+		const gchar *lang;
+		GType gdatype;
+
+		gdatype = gda_parameter_get_gda_type (param);
+#ifdef HAVE_LC_MESSAGES
+		lang = setlocale (LC_MESSAGES, NULL);
+#else
+		lang = setlocale (LC_CTYPE, NULL);
+#endif
+		while (vnode) {
+			if (xmlNodeIsText (vnode)) {
+				vnode = vnode->next;
+				continue;
+			}
+
+			if (strcmp (vnode->name, "gda_value")) {
+				vnode = vnode->next;
+				continue;
+			}
+
+			/* don't care about entries for the wrong locale */
+			this_lang = xmlGetProp (vnode, "lang");
+			if (this_lang && strncmp (this_lang, lang, strlen (this_lang))) {
+				g_free (this_lang);
+				vnode = vnode->next;
+				continue;
+			}
+			
+			isnull = xmlGetProp (vnode, "isnull");
+			if (isnull) {
+				if ((*isnull == 'f') || (*isnull == 'F')) {
+					xmlFree (isnull);
+					isnull = NULL;
+				}
+			}
+			
+			if (!isnull) {
+				GValue *value;
+
+				value = g_new0 (GValue, 1);
+				if (! gda_value_set_from_string (value, xmlNodeGetContent (vnode), gdatype)) {
+					/* error */
+					g_free (value);
+				}
+				else {
+					gda_parameter_set_value (param, value);
+					gda_value_free (value);
+				}
+			}
+			else {
+				gda_parameter_set_value (param, NULL);
+				xmlFree (isnull);
+			}
+			
+			vnode = vnode->next;
+		}
+	}
+}
+
+/**
+ * @dict:
+ * @prov:
+ * @cnc:
+ * @dbms_type:
+ * @gda_type:
+ * @created: a place to return if the data type has been created and is thus considered as a custom
+ *           data type. Can't be %NULL!
+ *
+ * Finds or creates anew GdaDictType if possible. if @created is returned as TRUE, then the
+ * caller of this function _does_ have a reference on the returned object.
+ *
+ * Returns: a #GdaDictType, or %NULL if it was not possible to find and create one
+ */
+GdaDictType *
+utility_find_or_create_data_type (GdaDict *dict, GdaServerProvider *prov, GdaConnection *cnc, 
+				  const gchar *dbms_type, const gchar *gda_type, gboolean *created)
+{
+	GdaDictType *dtype = NULL;
+
+	g_return_val_if_fail (created, NULL);
+	g_return_val_if_fail (!dict || GDA_IS_DICT (dict), NULL);
+	g_return_val_if_fail (!prov || GDA_IS_SERVER_PROVIDER (prov), NULL);
+	g_return_val_if_fail (!cnc || GDA_IS_CONNECTION (cnc), NULL);
+
+	*created = FALSE;
+	if (dbms_type)
+		dtype = gda_dict_get_dict_type_by_name (ASSERT_DICT (dict), dbms_type);
+
+	if (!dtype) {
+		if (gda_type) {
+			GType gtype;
+			
+			gtype = gda_type_from_string (gda_type);
+			if (prov) {
+				const gchar *deftype;
+				
+				deftype = gda_server_provider_get_default_dbms_type (prov, cnc, gtype);
+				if (deftype)
+					dtype = gda_dict_get_dict_type_by_name (ASSERT_DICT (dict), deftype);
+			}	
+			
+			if (!dtype) {
+				/* create a GdaDictType for that 'gda-type' */
+				dtype = GDA_DICT_TYPE (gda_dict_type_new (ASSERT_DICT (dict)));
+				gda_dict_type_set_sqlname (dtype, gda_type);
+				gda_dict_type_set_gda_type (dtype, gtype);
+				gda_dict_declare_object (ASSERT_DICT (dict), (GdaObject *) dtype);
+				*created = TRUE;
+			}
+		}
+	}
+
+	return dtype;
+}
