@@ -1,5 +1,5 @@
 /* GDA FireBird Provider
- * Copyright (C) 1998-2002 The GNOME Foundation
+ * Copyright (C) 1998 - 2006 The GNOME Foundation
  *
  * AUTHORS:
  *         Albi Jeronimo <jeronimoalbi@yahoo.com.ar>
@@ -20,13 +20,17 @@
  * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#include <libgda/gda-intl.h>
+#include <glib/gi18n-lib.h>
 #include <libgda/gda-data-model-array.h>
+#include <libgda/gda-data-model-private.h>
 #include <libgda/gda-command.h>
 #include <glib/gprintf.h>
 #include <string.h>
 #include "gda-firebird-provider.h"
 #include "gda-firebird-recordset.h"
+#include "gda-firebird-blob.h"
+
+#include <libgda/sql-delimiter/gda-sql-delimiter.h>
 
 static void 		gda_firebird_provider_class_init (GdaFirebirdProviderClass *klass);
 static void 		gda_firebird_provider_init (GdaFirebirdProvider *provider,
@@ -48,12 +52,6 @@ static const gchar 	*gda_firebird_provider_get_database (GdaServerProvider *prov
 static gboolean		gda_firebird_provider_change_database (GdaServerProvider *provider,
 							       GdaConnection *cnc,
 							       const gchar *name);
-static gboolean 	gda_firebird_provider_create_database_cnc (GdaServerProvider *provider,
-					  		           GdaConnection *cnc,
-							           const gchar *name);
-static gboolean 	gda_firebird_provider_drop_database (GdaServerProvider *provider,
-							     GdaConnection *cnc,
-							     const gchar *name);
 static GList 		*gda_firebird_provider_execute_command (GdaServerProvider *provider,
 								GdaConnection *cnc,
 								GdaCommand *cmd,
@@ -75,6 +73,8 @@ static GdaDataModel	*gda_firebird_provider_get_schema (GdaServerProvider *provid
 							   GdaConnectionSchema schema,
 							   GdaParameterList *params);
 
+static GdaBlob          *gda_firebird_provider_create_blob (GdaServerProvider *provider,
+							    GdaConnection *cnc);
 
 static GObjectClass *parent_class = NULL;
 
@@ -91,19 +91,34 @@ gda_firebird_provider_class_init (GdaFirebirdProviderClass *klass)
 
 	object_class->finalize = gda_firebird_provider_finalize;
 	provider_class->get_version = gda_firebird_provider_get_version;
+	provider_class->get_server_version = gda_firebird_provider_get_server_version;
+	provider_class->get_info = NULL;
+	provider_class->supports_feature = gda_firebird_provider_supports;
+	provider_class->get_schema = gda_firebird_provider_get_schema;
+
+	provider_class->get_data_handler = NULL;
+	provider_class->string_to_value = NULL;
+	provider_class->get_def_dbms_type = NULL;
+
 	provider_class->open_connection = gda_firebird_provider_open_connection;
 	provider_class->close_connection = gda_firebird_provider_close_connection;
-	provider_class->get_server_version = gda_firebird_provider_get_server_version;
 	provider_class->get_database = gda_firebird_provider_get_database;
 	provider_class->change_database = gda_firebird_provider_change_database;
-	provider_class->create_database_cnc = gda_firebird_provider_create_database_cnc;
-	provider_class->drop_database = gda_firebird_provider_drop_database;
+
+	provider_class->supports_operation = NULL;
+        provider_class->create_operation = NULL;
+        provider_class->render_operation = NULL;
+        provider_class->perform_operation = NULL;
+
 	provider_class->execute_command = gda_firebird_provider_execute_command;
+	provider_class->get_last_insert_id = NULL;
+
 	provider_class->begin_transaction = gda_firebird_provider_begin_transaction;
 	provider_class->commit_transaction = gda_firebird_provider_commit_transaction;
 	provider_class->rollback_transaction = gda_firebird_provider_rollback_transaction;
-	provider_class->supports = gda_firebird_provider_supports;
-	provider_class->get_schema = gda_firebird_provider_get_schema;
+	
+	provider_class->create_blob = gda_firebird_provider_create_blob;
+	provider_class->fetch_blob = NULL;
 }
 
 static void
@@ -208,20 +223,20 @@ fb_get_types (GdaConnection *cnc,
 		const gchar *name;
 		const gchar *owner;
 		const gchar *comments;
-		GdaValueType type;
+		GType type;
 	} types[] = {
-		{ "blob", "", "Binary large object (blob)", GDA_VALUE_TYPE_BINARY },
-		{ "char", "", "Fixed length string", GDA_VALUE_TYPE_STRING },
-		{ "varchar", "", "Variable length string", GDA_VALUE_TYPE_STRING },
-		{ "date", "", "Date", GDA_VALUE_TYPE_DATE },
-		{ "time", "", "Time", GDA_VALUE_TYPE_TIME },
-		{ "timestamp", "", "Time stamp", GDA_VALUE_TYPE_TIMESTAMP },
-		{ "smallint", "", "Signed short integer", GDA_VALUE_TYPE_SMALLINT },
-		{ "integer", "", "Signed long integer (longword)", GDA_VALUE_TYPE_INTEGER },
-		{ "decimal", "", "Decimal number", GDA_VALUE_TYPE_NUMERIC },
-		{ "numeric", "", "Decimal number", GDA_VALUE_TYPE_NUMERIC },
-		{ "float", "", "Single precision number", GDA_VALUE_TYPE_SINGLE },
-		{ "double precision", "", "Double precision number", GDA_VALUE_TYPE_DOUBLE }
+		{ "blob", "", "Binary large object (blob)", GDA_TYPE_BINARY },
+		{ "char", "", "Fixed length string", G_TYPE_STRING },
+		{ "varchar", "", "Variable length string", G_TYPE_STRING },
+		{ "date", "", "Date", G_TYPE_DATE },
+		{ "time", "", "Time", GDA_TYPE_TIME },
+		{ "timestamp", "", "Time stamp", GDA_TYPE_TIMESTAMP },
+		{ "smallint", "", "Signed short integer", GDA_TYPE_SHORT },
+		{ "integer", "", "Signed long integer (longword)", G_TYPE_INT },
+		{ "decimal", "", "Decimal number", GDA_TYPE_NUMERIC },
+		{ "numeric", "", "Decimal number", GDA_TYPE_NUMERIC },
+		{ "float", "", "Single precision number", G_TYPE_FLOAT },
+		{ "double precision", "", "Double precision number", G_TYPE_DOUBLE }
 	};
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
@@ -236,14 +251,22 @@ fb_get_types (GdaConnection *cnc,
 	/* Fill the recordset with each of the values from the array defined above */
 	for (i = 0; i < (sizeof (types) / sizeof (types[0])); i++) {
 		value_list = NULL;
+		GValue *tmpval;
 		
-		value_list = g_list_append (value_list, gda_value_new_string (types[i].name));
-		value_list = g_list_append (value_list, gda_value_new_string (types[i].owner));
-		value_list = g_list_append (value_list, gda_value_new_string (types[i].comments));
-		value_list = g_list_append (value_list, gda_value_new_type (types[i].type));
+		g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), types[i].name);
+		value_list = g_list_append (value_list, tmpval);
+
+		g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), types[i].owner);
+		value_list = g_list_append (value_list, tmpval);
+
+		g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), types[i].comments);
+		value_list = g_list_append (value_list, tmpval);
+
+		g_value_set_ulong (tmpval = gda_value_new (G_TYPE_ULONG), types[i].type);
+		value_list = g_list_append (value_list, tmpval);
 
 		/* Add values to row */
-		gda_data_model_append_values (GDA_DATA_MODEL (recset), value_list);
+		gda_data_model_append_values (GDA_DATA_MODEL (recset), value_list, NULL);
 
 		/* Free temporary list of values */
 		g_list_foreach (value_list, (GFunc) gda_value_free, NULL);
@@ -273,7 +296,7 @@ fb_get_tables (GdaConnection *cnc,
 	GdaDataModel *recset = NULL;
 	GdaTransaction *transaction;
 	GdaCommand *command;
-	GdaValue *value;
+	GValue *value;
 	GdaRow *row;
 	GdaParameter *par = NULL;
 	gchar *sql, *sql_cond;
@@ -291,7 +314,7 @@ fb_get_tables (GdaConnection *cnc,
 
 		/* Initialize parameter values */
 		if (par)
-			systables = gda_value_get_boolean ((GdaValue *) gda_parameter_get_value (par));
+			systables = g_value_get_boolean ((GValue *) gda_parameter_get_value (par));
 	
 		/* Evaluate statement conditions */	
 		if (show_views) {
@@ -313,7 +336,7 @@ fb_get_tables (GdaConnection *cnc,
 		/* Execute statement */
 		command = gda_command_new (sql, GDA_COMMAND_TYPE_SQL, GDA_COMMAND_OPTION_STOP_ON_ERRORS);
 		gda_command_set_transaction (command, transaction);		
-		reclist = gda_connection_execute_command (cnc, command, NULL);
+		reclist = gda_connection_execute_command_l (cnc, command, NULL, NULL);
 
 		if (reclist) {
 			recset = gda_data_model_array_new (4);
@@ -323,25 +346,25 @@ fb_get_tables (GdaConnection *cnc,
 			gda_data_model_set_column_title (recset, 3, _("Definition"));			
 			for (i = 0; i < gda_data_model_get_n_rows (GDA_DATA_MODEL (reclist->data)); i++) {
 				value_list = NULL;
+				GValue *tmpval;
 
 				/* Get name of table */
 				row = (GdaRow *) gda_data_model_get_row (GDA_DATA_MODEL (reclist->data), i);
 				value = gda_row_get_value (row, 0);
-				value_list = g_list_append (
-						value_list,
-						gda_value_new_string ((gchar *) gda_value_get_string (value)));
+				value_list = g_list_append (value_list, gda_value_copy (value));
 
 				/* Get owner of table */
 				value = gda_row_get_value (row, 1);
-				value_list = g_list_append (
-						value_list,
-						gda_value_new_string ((gchar *) gda_value_get_string (value)));
+				value_list = g_list_append (value_list, gda_value_copy (value));
 				
 				/* FIXME: Set correct values */
-				value_list = g_list_append (value_list, gda_value_new_string (""));
-				value_list = g_list_append (value_list, gda_value_new_string (""));
+				g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), "");
+				value_list = g_list_append (value_list, tmpval);
 
-				gda_data_model_append_values (recset, value_list);
+				g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), "");
+				value_list = g_list_append (value_list, tmpval);
+
+				gda_data_model_append_values (recset, value_list, NULL);
 				g_list_foreach (value_list, (GFunc) gda_value_free, NULL);
 				g_list_free (value_list);
 			}
@@ -366,36 +389,36 @@ fb_get_tables (GdaConnection *cnc,
  *  Function convert a string containing the name of Firebird's
  *  data type to it's corresponding gda data type.
  *
- *  Returns: The GdaValueType corresponding to @name Firebird's
+ *  Returns: The GType corresponding to @name Firebird's
  *           type
  */
-static GdaValueType
+static GType
 fb_type_name_to_gda_type (const gchar *name)
 {
 	if (!strcmp (name, "CHAR") || !strcmp (name, "VARCHAR"))
-		return GDA_VALUE_TYPE_STRING;
+		return G_TYPE_STRING;
 	else if (!strcmp (name, "SMALLINT"))
-		return GDA_VALUE_TYPE_SMALLINT;
+		return GDA_TYPE_SHORT;
 	else if (!strcmp (name, "NUMERIC") || !strcmp (name, "DECIMAL"))
-		return GDA_VALUE_TYPE_NUMERIC;
+		return GDA_TYPE_NUMERIC;
 	else if (!strcmp (name, "INTEGER"))
-		return GDA_VALUE_TYPE_INTEGER;
+		return G_TYPE_INT;
 	else if (!strcmp (name, "BLOB"))
-		return GDA_VALUE_TYPE_BLOB;
+		return GDA_TYPE_BLOB;
 	else if (!strcmp (name, "TIMESTAMP"))
-		return GDA_VALUE_TYPE_TIMESTAMP;
+		return GDA_TYPE_TIMESTAMP;
 	else if (!strcmp (name, "TIME"))
-		return GDA_VALUE_TYPE_TIME;
+		return GDA_TYPE_TIME;
 	else if (!strcmp (name, "DATE"))
-		return GDA_VALUE_TYPE_DATE;
+		return G_TYPE_DATE;
 	else if (!strcmp (name, "INT64"))
-		return GDA_VALUE_TYPE_NUMERIC;
+		return GDA_TYPE_NUMERIC;
 	else if (!strcmp (name, "FLOAT"))
-		return GDA_VALUE_TYPE_SINGLE;
+		return G_TYPE_FLOAT;
 	else if (!strcmp (name, "DOUBLE"))
-		return GDA_VALUE_TYPE_DOUBLE;
+		return G_TYPE_DOUBLE;
 
-	return GDA_VALUE_TYPE_STRING;
+	return G_TYPE_STRING;
 }
 
 
@@ -416,7 +439,7 @@ fb_set_index_field_metadata (GdaConnection *cnc,
 	gchar *sql, *field_name, *recset_field_name;
 	gint i,j;
 	GList *reclist;
-	GdaValue *value, *recset_value;
+	GValue *value, *recset_value;
 	
 	g_return_if_fail (GDA_IS_CONNECTION (cnc));
 	g_return_if_fail (GDA_IS_DATA_MODEL (recset));
@@ -433,49 +456,49 @@ fb_set_index_field_metadata (GdaConnection *cnc,
 				table_name);
 		command = gda_command_new (sql, GDA_COMMAND_TYPE_SQL, GDA_COMMAND_OPTION_STOP_ON_ERRORS);
 		gda_command_set_transaction (command, transaction);
-		reclist = gda_connection_execute_command (cnc, command, NULL);
+		reclist = gda_connection_execute_command_l (cnc, command, NULL, NULL);
 		if (reclist) {
 			gda_data_model_freeze (recset);
 			
 			/* For each index or unique field in statement */
 			for (i = 0; i < gda_data_model_get_n_rows (GDA_DATA_MODEL (reclist->data)); i++) {
-				value = (GdaValue *) gda_data_model_get_value_at (GDA_DATA_MODEL (reclist->data), 0, i);
-				field_name = (gchar *) gda_value_get_string (value);
+				value = (GValue *) gda_data_model_get_value_at (GDA_DATA_MODEL (reclist->data), 0, i);
+				field_name = (gchar *) g_value_get_string (value);
 				j = -1;
 
 				/* Find column in @recset by name */
 				do {
 					j++;
-					recset_value = (GdaValue *) gda_data_model_get_value_at (recset, 0, j);
-					recset_field_name = (gchar *) gda_value_get_string (recset_value);
+					recset_value = (GValue *) gda_data_model_get_value_at (recset, 0, j);
+					recset_field_name = (gchar *) g_value_get_string (recset_value);
 				} while (strcmp (recset_field_name, field_name) && (j < gda_data_model_get_n_rows (recset)));
 				
 				/* Set recset values */
 				if (!strcmp (recset_field_name, field_name)) {
 					
 					/* Get primary key value from statement */
-					value = (GdaValue *) gda_data_model_get_value_at (GDA_DATA_MODEL (reclist->data),
+					value = (GValue *) gda_data_model_get_value_at (GDA_DATA_MODEL (reclist->data),
 											  2,
 											  i);
 					
-					/* Get recorset GdaValue for primary key (position 5) */
-					recset_value = (GdaValue *) gda_data_model_get_value_at (recset, 5, j);
+					/* Get recorset GValue for primary key (position 5) */
+					recset_value = (GValue *) gda_data_model_get_value_at (recset, 5, j);
 
 					/* Set Primary Key mark */
-					gda_value_set_boolean (recset_value, (!strcmp (
-										gda_value_get_string (value),
+					g_value_set_boolean (recset_value, (!strcmp (
+										g_value_get_string (value),
 										"PRIMARY KEY")));
 					
 					/* Get unique index value from statement */
-					value = (GdaValue *) gda_data_model_get_value_at (GDA_DATA_MODEL (reclist->data),
+					value = (GValue *) gda_data_model_get_value_at (GDA_DATA_MODEL (reclist->data),
 											  1,
 											  i);
 					
-					/* Get recorset GdaValue for unique index (position 6) */
-					recset_value = (GdaValue *) gda_data_model_get_value_at (recset, 6, j);
+					/* Get recorset GValue for unique index (position 6) */
+					recset_value = (GValue *) gda_data_model_get_value_at (recset, 6, j);
 
 					/* Set Unique Index mark */
-					gda_value_set_boolean (recset_value, (gda_value_get_smallint (value) == 1));
+					g_value_set_boolean (recset_value, (gda_value_get_short (value) == 1));
 				}
 			}
 			
@@ -498,90 +521,78 @@ fb_set_index_field_metadata (GdaConnection *cnc,
  *  Fill a list with metadata values from @row.
  *  This function is called from #fb_get_fields_metadata.
  *
- *  Returns: A GList of GdaValues
+ *  Returns: A GList of GValues
  */
 static GList *
 fb_set_field_metadata (GdaRow *row)
 {
 	GList *value_list = NULL;
-	GdaValue *value;
+	GValue *value;
 	gchar *the_value, *tmp_value;
 	gshort scale, short_value;
+	GValue *tmpval;
+	GType type;
 
 	g_return_val_if_fail (row != NULL, NULL);
 
 	/* Set field name  */
 	value = gda_row_get_value (row, 0);
-	the_value = (gchar *) gda_value_get_string (value);
-	value_list = g_list_append (value_list,	gda_value_new_string (the_value));
-	
-	/* Get Scale (number of decimals) */
-	value = gda_row_get_value (row, 4);
-	if (!gda_value_is_null (value))
-		scale = gda_value_get_smallint (value);
-	else
-		scale = 0;
-	
+	value_list = g_list_append (value_list,	gda_value_copy (value));
+		
 	/* Set data type */
 	value = gda_row_get_value (row, 1);
-	tmp_value = (gchar *) gda_value_get_string (value);
+	tmp_value = (gchar *) g_value_get_string (value);
 	tmp_value = g_strchomp (tmp_value);
 	the_value = g_ascii_strdown (tmp_value, -1);
 	g_free (tmp_value);
-	
-	/* Set correct value name */
-	if (!strcmp (the_value, "long") && (scale < 0))
-		value_list = g_list_append (value_list, gda_value_new_string ("decimal"));
-	else if (!strcmp (the_value, "long"))
-		value_list = g_list_append (value_list, gda_value_new_string ("integer"));
-	else if (!strcmp (the_value, "int64"))
-		value_list = g_list_append (value_list, gda_value_new_string ("numeric"));
-	else if (!strcmp (the_value, "varying"))
-		value_list = g_list_append (value_list, gda_value_new_string ("varchar"));
-	else if (!strcmp (the_value, "text"))
-		value_list = g_list_append (value_list, gda_value_new_string ("char"));
-	else if (!strcmp (the_value, "short"))
-		value_list = g_list_append (value_list, gda_value_new_string ("smallint"));
-	else if (!strcmp (the_value, "double"))
-		value_list = g_list_append (value_list, gda_value_new_string ("double precision"));
-	else
-		value_list = g_list_append (value_list, gda_value_new_string (the_value));
+	g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), the_value);
+	value_list = g_list_append (value_list, tmpval);
 
 	/* Set size */
 	short_value = 0;
-	if (fb_type_name_to_gda_type (the_value) == GDA_VALUE_TYPE_STRING) {
+	if (fb_type_name_to_gda_type (the_value) == G_TYPE_STRING) {
 		value = gda_row_get_value (row, 2);
-		short_value = gda_value_get_smallint (value);
+		short_value = gda_value_get_short (value);
 	}
-	else if (fb_type_name_to_gda_type (the_value) == GDA_VALUE_TYPE_NUMERIC) {
+	else if (fb_type_name_to_gda_type (the_value) == GDA_TYPE_NUMERIC) {
 		value = gda_row_get_value (row, 3);
-		short_value = gda_value_get_smallint (value);
+		short_value = gda_value_get_short (value);
 	}
-	value_list = g_list_append (value_list, gda_value_new_integer (short_value));
+	g_value_set_int (tmpval = gda_value_new (G_TYPE_INT), short_value);
+	value_list = g_list_append (value_list, tmpval);
 
-	/* Set scale */	
-	value_list = g_list_append (value_list, gda_value_new_integer ((scale * -1)));
+	/* Set scale (number of decimals) */
+	value = gda_row_get_value (row, 4);
+	if (!gda_value_is_null (value))
+		scale = gda_value_get_short (value);
+	else
+		scale = 0;
+	g_value_set_int (tmpval = gda_value_new (G_TYPE_INT), scale * -1);
+	value_list = g_list_append (value_list, tmpval);
 
 	/* Set not null indicator */
 	value = gda_row_get_value (row, 5);
 	if (!gda_value_is_null (value))
-		value_list = g_list_append (
-				value_list,
-				gda_value_new_boolean ((gda_value_get_smallint (value) == 1)));
+		g_value_set_boolean (tmpval = gda_value_new (G_TYPE_BOOLEAN), (gda_value_get_short (value) == 1));
 	else
-		value_list = g_list_append (value_list, gda_value_new_boolean (FALSE));
+		g_value_set_boolean (tmpval = gda_value_new (G_TYPE_BOOLEAN), FALSE);
+	value_list = g_list_append (value_list, tmpval);
 
 	/* Primary key ? (real value set in #fb_set_index_field_metadata) */
-	value_list = g_list_append (value_list, gda_value_new_boolean (FALSE));
+	g_value_set_boolean (tmpval = gda_value_new (G_TYPE_BOOLEAN), FALSE);
+	value_list = g_list_append (value_list, tmpval);
 
 	/* Unique key ? (real value set in #fb_set_index_field_metadata) */
-	value_list = g_list_append (value_list, gda_value_new_boolean (FALSE));
+	g_value_set_boolean (tmpval = gda_value_new (G_TYPE_BOOLEAN), FALSE);
+	value_list = g_list_append (value_list, tmpval);
 
 	/* FIXME: References */
-	value_list = g_list_append (value_list, gda_value_new_string (""));
+	tmpval = gda_value_new_null ();
+	value_list = g_list_append (value_list, tmpval);
 
 	/* FIXME: Default value */
-	value_list = g_list_append (value_list, gda_value_new_string (""));
+	tmpval = gda_value_new_null ();
+	value_list = g_list_append (value_list, tmpval);
 
 	return value_list;
 }
@@ -610,17 +621,17 @@ fb_get_fields_metadata (GdaConnection *cnc,
 	const gchar *table_name;
 	struct {
 	  	const gchar *name;
-		GdaValueType type;
+		GType type;
 	} fields_desc[] = {
-		{ N_("Field name")      , GDA_VALUE_TYPE_STRING  },
-		{ N_("Data type")       , GDA_VALUE_TYPE_STRING  },
-		{ N_("Size")            , GDA_VALUE_TYPE_INTEGER },
-		{ N_("Scale")           , GDA_VALUE_TYPE_INTEGER },
-		{ N_("Not null?")       , GDA_VALUE_TYPE_BOOLEAN },
-		{ N_("Primary key?")    , GDA_VALUE_TYPE_BOOLEAN },
-		{ N_("Unique index?")   , GDA_VALUE_TYPE_BOOLEAN },
-		{ N_("References")      , GDA_VALUE_TYPE_STRING  },
-		{ N_("Default value")   , GDA_VALUE_TYPE_STRING  }
+		{ N_("Field name")      , G_TYPE_STRING  },
+		{ N_("Data type")       , G_TYPE_STRING  },
+		{ N_("Size")            , G_TYPE_INT },
+		{ N_("Scale")           , G_TYPE_INT },
+		{ N_("Not null?")       , G_TYPE_BOOLEAN },
+		{ N_("Primary key?")    , G_TYPE_BOOLEAN },
+		{ N_("Unique index?")   , G_TYPE_BOOLEAN },
+		{ N_("References")      , G_TYPE_STRING  },
+		{ N_("Default value")   , G_TYPE_STRING  }
 	};
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
@@ -628,14 +639,14 @@ fb_get_fields_metadata (GdaConnection *cnc,
 	
 	par = gda_parameter_list_find (params, "name");
 	if (!par) {
-		gda_connection_add_error_string (cnc,
+		gda_connection_add_event_string (cnc,
 				_("Table name is needed but none specified in parameter list"));
 		return NULL;
 	}
 
-	table_name = gda_value_get_string ((GdaValue *) gda_parameter_get_value (par));
+	table_name = g_value_get_string ((GValue *) gda_parameter_get_value (par));
 	if (!table_name) {
-		gda_connection_add_error_string (cnc,
+		gda_connection_add_event_string (cnc,
 				_("Table name is needed but none specified in parameter list"));
 		return NULL;
 	}
@@ -652,7 +663,7 @@ fb_get_fields_metadata (GdaConnection *cnc,
 				table_name);
 		command = gda_command_new (sql, GDA_COMMAND_TYPE_SQL, GDA_COMMAND_OPTION_STOP_ON_ERRORS);
 		gda_command_set_transaction (command, transaction);
-		reclist = gda_connection_execute_command (cnc, command, NULL);
+		reclist = gda_connection_execute_command_l (cnc, command, NULL, NULL);
 		if (reclist) {
 			/* Create and fill recordset to be returned */
 			recset = (GdaDataModelArray *) gda_data_model_array_new (9);
@@ -666,7 +677,7 @@ fb_get_fields_metadata (GdaConnection *cnc,
 				
 				/* Set field metdata for row, then append to recordset */
 				value_list = fb_set_field_metadata (row);
-				gda_data_model_append_values (GDA_DATA_MODEL (recset), value_list);
+				gda_data_model_append_values (GDA_DATA_MODEL (recset), value_list, NULL);
 
 				/* Free temp list of values */
 				g_list_foreach (value_list, (GFunc) gda_value_free, NULL);
@@ -693,25 +704,33 @@ fb_add_aggregate_row (GdaDataModelArray *recset,
 		      const gchar *comments)
 {
 	GList *list;
+	GValue *tmpval;
 
 	g_return_if_fail (GDA_IS_DATA_MODEL_ARRAY (recset));
 
 	/* 1st the name */
-	list = g_list_append (NULL, gda_value_new_string (str));
+	g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), str);
+	list = g_list_append (NULL, tmpval);
 	/* 2nd the unique id */
-	list = g_list_append (list, gda_value_new_string (str));
+	g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), str);
+	list = g_list_append (list, tmpval);
 	/* 3rd the owner */
-	list = g_list_append (list, gda_value_new_string (NULL));
+	tmpval = gda_value_new_null ();
+	list = g_list_append (list, tmpval);
 	/* 4th the comments */
-	list = g_list_append (list, gda_value_new_string (comments));
+	g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), comments);
+	list = g_list_append (list, tmpval);
 	/* 5th the return type */
-	list = g_list_append (list, gda_value_new_string (_("UNKNOWN")));
+	g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), _("UNKNOWN"));
+	list = g_list_append (list, tmpval);
 	/* 6th the argument type */
-	list = g_list_append (list, gda_value_new_string (_("UNKNOWN")));
+	g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), _("UNKNOWN"));
+	list = g_list_append (list, tmpval);
 	/* 7th the SQL definition */
-	list = g_list_append (list, gda_value_new_string (NULL));
+	tmpval = gda_value_new_null ();
+	list = g_list_append (list, tmpval);
 
-	gda_data_model_append_values (GDA_DATA_MODEL (recset), list);
+	gda_data_model_append_values (GDA_DATA_MODEL (recset), list, NULL);
 
 	g_list_foreach (list, (GFunc) gda_value_free, NULL);
 	g_list_free (list);
@@ -809,7 +828,7 @@ void
 gda_firebird_connection_make_error (GdaConnection *cnc,
 				    const gint statement_type)
 {
-	GdaError *error;
+	GdaConnectionEvent *error;
 	GdaFirebirdConnection *fcnc;
 	gchar *description;
 
@@ -817,16 +836,16 @@ gda_firebird_connection_make_error (GdaConnection *cnc,
 
 	fcnc = g_object_get_data (G_OBJECT (cnc), CONNECTION_DATA);
 	if (!fcnc) {
-		gda_connection_add_error_string (cnc, _("Invalid Firebird handle"));
+		gda_connection_add_event_string (cnc, _("Invalid Firebird handle"));
 		return;
 	}
 	
-	error = gda_error_new ();
-	gda_error_set_number (error, isc_sqlcode (fcnc->status));
-	gda_error_set_source (error, "[GDA Firebird]");
+	error = gda_connection_event_new (GDA_CONNECTION_EVENT_ERROR);
+	gda_connection_event_set_code (error, isc_sqlcode (fcnc->status));
+	gda_connection_event_set_source (error, "[GDA Firebird]");
 	description = fb_sqlerror_get_description (fcnc, statement_type);
-	gda_error_set_description (error, description);
-	gda_connection_add_error (cnc, error);	
+	gda_connection_event_set_description (error, description);
+	gda_connection_add_event (cnc, error);	
 	g_free (description);
 }
 																								
@@ -855,7 +874,7 @@ gda_firebird_provider_open_connection (GdaServerProvider *provider,
 	/* get parameters */
 	fb_db = (gchar *) gda_quark_list_find (params, "DATABASE");
 	if (!fb_db) {
-		gda_connection_add_error_string (cnc, _("No database specified"));
+		gda_connection_add_event_string (cnc, _("No database specified"));
 		return FALSE;
 	}
 
@@ -934,7 +953,7 @@ gda_firebird_provider_close_connection (GdaServerProvider *provider,
 	
 	fcnc = g_object_get_data (G_OBJECT (cnc), CONNECTION_DATA);
 	if (!fcnc) {
-		gda_connection_add_error_string (cnc, _("Invalid Firebird handle"));
+		gda_connection_add_event_string (cnc, _("Invalid Firebird handle"));
 		return FALSE;
 	}
 
@@ -961,7 +980,7 @@ gda_firebird_provider_get_server_version (GdaServerProvider *provider,
 
 	fcnc = g_object_get_data (G_OBJECT (cnc), CONNECTION_DATA);
 	if (!fcnc) {
-		gda_connection_add_error_string (cnc, _("Invalid Firebird handle"));
+		gda_connection_add_event_string (cnc, _("Invalid Firebird handle"));
 		return FALSE;
 	}
 
@@ -980,7 +999,7 @@ gda_firebird_provider_get_database (GdaServerProvider *provider,
 
 	fcnc = g_object_get_data (G_OBJECT (cnc), CONNECTION_DATA);
 	if (!fcnc) {
-		gda_connection_add_error_string (cnc, _("Invalid Firebird handle"));
+		gda_connection_add_event_string (cnc, _("Invalid Firebird handle"));
 		return NULL;
 	}
 
@@ -1016,7 +1035,7 @@ gda_firebird_provider_create_database_cnc (GdaServerProvider *provider,
 	
 	fcnc = g_object_get_data (G_OBJECT (cnc), CONNECTION_DATA);
 	if (!fcnc) {
-		gda_connection_add_error_string (cnc, _("Invalid Firebird handle"));
+		gda_connection_add_event_string (cnc, _("Invalid Firebird handle"));
 		return FALSE;
 	}
 	
@@ -1062,9 +1081,9 @@ gda_firebird_provider_create_database_cnc (GdaServerProvider *provider,
 
 static GList *
 gda_firebird_provider_run_sql (GList *reclist,
-				GdaConnection *cnc,
-				isc_tr_handle *ftr,
-				const gchar *sql)
+			       GdaConnection *cnc,
+			       isc_tr_handle *ftr,
+			       const gchar *sql)
 {
 	GdaFirebirdRecordset *recset;
 	gchar **arr;
@@ -1075,13 +1094,13 @@ gda_firebird_provider_run_sql (GList *reclist,
 	recset = NULL;
 
 	/* parse SQL string, which can contain several commands, separated by ';' */
-	arr = g_strsplit (sql, ";", 0);
+	arr = gda_delimiter_split_sql (sql);
 	if (arr) {
 		while (arr[n]) {
 			recset = gda_firebird_recordset_new (cnc, ftr, sql);
   			if (GDA_IS_FIREBIRD_RECORDSET (recset)) {
-				gda_data_model_set_command_text ((GdaDataModel*) recset, arr[n]);
-				gda_data_model_set_command_type ((GdaDataModel*) recset, GDA_COMMAND_TYPE_SQL);
+				g_object_set (G_OBJECT (recset), "command_text", arr[n],
+					      "command_type", GDA_COMMAND_TYPE_SQL, NULL);
 				reclist = g_list_append (reclist, recset);
 			}
 			
@@ -1094,11 +1113,11 @@ gda_firebird_provider_run_sql (GList *reclist,
 	return reclist;
 }
 
-/* drop_database handler for the GdaFirebirdProvider class */
+/* drop_database_cnc handler for the GdaFirebirdProvider class */
 static gboolean
-gda_firebird_provider_drop_database (GdaServerProvider *provider,
-				     GdaConnection *cnc,
-				     const gchar *name)
+gda_firebird_provider_drop_database_cnc (GdaServerProvider *provider,
+					 GdaConnection *cnc,
+					 const gchar *name)
 {
 	GdaFirebirdConnection *fcnc;
 	isc_db_handle db_handle = NULL;
@@ -1109,7 +1128,7 @@ gda_firebird_provider_drop_database (GdaServerProvider *provider,
 	/* Get current firebird connection */
 	fcnc = g_object_get_data (G_OBJECT (cnc), CONNECTION_DATA);
 	if (!fcnc) {
-		gda_connection_add_error_string (cnc, _("Invalid Firebird handle"));
+		gda_connection_add_event_string (cnc, _("Invalid Firebird handle"));
 		return FALSE;
 	}
 	
@@ -1204,7 +1223,7 @@ gda_firebird_provider_begin_transaction (GdaServerProvider *provider,
 
 	fcnc = g_object_get_data (G_OBJECT (cnc), CONNECTION_DATA);
 	if (!fcnc) {
-		gda_connection_add_error_string (cnc, _("Invalid Firebird handle"));
+		gda_connection_add_event_string (cnc, _("Invalid Firebird handle"));
 		return FALSE;
 	}
 
@@ -1238,13 +1257,13 @@ gda_firebird_provider_commit_transaction (GdaServerProvider *provider,
 
 	fcnc = g_object_get_data (G_OBJECT (cnc), CONNECTION_DATA);
 	if (!fcnc) {
-		gda_connection_add_error_string (cnc, _("Invalid Firebird handle"));
+		gda_connection_add_event_string (cnc, _("Invalid Firebird handle"));
 		return FALSE;
 	}
 
 	ftr = g_object_get_data (G_OBJECT (xaction), TRANSACTION_DATA);
 	if (!ftr) {
-		gda_connection_add_error_string (cnc, _("Invalid transaction handle"));
+		gda_connection_add_event_string (cnc, _("Invalid transaction handle"));
 		return FALSE;
 	}
 
@@ -1276,13 +1295,13 @@ gda_firebird_provider_rollback_transaction (GdaServerProvider *provider,
 
 	fcnc = g_object_get_data (G_OBJECT (cnc), CONNECTION_DATA);
 	if (!fcnc) {
-		gda_connection_add_error_string (cnc, _("Invalid Firebird handle"));
+		gda_connection_add_event_string (cnc, _("Invalid Firebird handle"));
 		return FALSE;
 	}
 
 	ftr = g_object_get_data (G_OBJECT (xaction), TRANSACTION_DATA);
 	if (!ftr) {
-		gda_connection_add_error_string (cnc, _("Invalid transaction handle"));
+		gda_connection_add_event_string (cnc, _("Invalid transaction handle"));
 		return FALSE;
 	}
 
@@ -1306,7 +1325,6 @@ gda_firebird_provider_supports (GdaServerProvider *provider,
 				GdaConnectionFeature feature)
 {
 	g_return_val_if_fail (GDA_IS_FIREBIRD_PROVIDER (provider), FALSE);
-	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
 
 	switch (feature) {
 		case GDA_CONNECTION_FEATURE_VIEWS:
@@ -1363,3 +1381,11 @@ gda_firebird_provider_get_schema (GdaServerProvider *provider,
 }
 
 
+static GdaBlob *
+gda_firebird_provider_create_blob (GdaServerProvider *provider, GdaConnection *cnc)
+{
+	g_return_val_if_fail (GDA_IS_SERVER_PROVIDER (provider), FALSE);
+	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
+
+	return gda_firebird_blob_new (cnc);
+}
