@@ -1,8 +1,9 @@
 /* GDA FreeTDS Provider
- * Copyright (C) 2002 The GNOME Foundation
+ * Copyright (C) 2002 - 2006 The GNOME Foundation
  *
  * AUTHORS:
  *         Holger Thon <holger.thon@gnome-db.org>
+ *         Vivien Malerba <malerba@gnome-db.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -33,6 +34,8 @@
 #include "gda-freetds.h"
 #include "gda-freetds-defs.h"
 
+#include <libgda/sql-delimiter/gda-sql-delimiter.h>
+
 #define PARENT_TYPE GDA_TYPE_SERVER_PROVIDER
 
 #define OBJECT_DATA_FREETDS_HANDLE "GDA_FreeTDS_FreeTDSHandle"
@@ -60,12 +63,6 @@ static const gchar *gda_freetds_provider_get_database (GdaServerProvider *provid
 static gboolean gda_freetds_provider_change_database (GdaServerProvider *provider,
                                                       GdaConnection *cnc,
                                                       const gchar *name);
-static gboolean gda_freetds_provider_create_database_cnc (GdaServerProvider *provider,
-                                                          GdaConnection *cnc,
-                                                          const gchar *name);
-static gboolean gda_freetds_provider_drop_database_cnc (GdaServerProvider *provider,
-							GdaConnection *cnc,
-							const gchar *name);
 static GList *gda_freetds_provider_execute_command (GdaServerProvider *provider,
                                                     GdaConnection *cnc,
                                                     GdaCommand *cmd,
@@ -483,32 +480,6 @@ gda_freetds_provider_change_database (GdaServerProvider *provider,
 	return ret;
 }
 
-static gboolean
-gda_freetds_provider_create_database_cnc (GdaServerProvider *provider,
-                                          GdaConnection *cnc,
-                                          const gchar *name)
-{
-	GdaFreeTDSProvider *tds_prov = (GdaFreeTDSProvider *) provider;
-
-	g_return_val_if_fail (GDA_IS_FREETDS_PROVIDER (tds_prov), FALSE);
-	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-
-	return FALSE;
-}
-
-static gboolean
-gda_freetds_provider_drop_database_cnc (GdaServerProvider *provider,
-					GdaConnection *cnc,
-					const gchar *name)
-{
-	GdaFreeTDSProvider *tds_prov = (GdaFreeTDSProvider *) provider;
-
-	g_return_val_if_fail (GDA_IS_FREETDS_PROVIDER (tds_prov), FALSE);
-	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	
-	return FALSE;
-}
-
 static GList
 *gda_freetds_provider_execute_command (GdaServerProvider *provider,
                                        GdaConnection *cnc,
@@ -640,11 +611,11 @@ static const gchar
 		if (model) {
 			if ((gda_data_model_get_n_columns (model) == 1)
 			    && (gda_data_model_get_n_rows (model) == 1)) {
-				GdaValue *value;
+				GValue *value;
 
-				value = (GdaValue *) gda_data_model_get_value_at
+				value = (GValue *) gda_data_model_get_value_at
 				                             (model, 0, 0);
-				tds_cnc->server_id = gda_value_stringify ((GdaValue *) value);
+				tds_cnc->server_id = gda_value_stringify ((GValue *) value);
 			}
 			g_object_unref (model);
 		}
@@ -669,8 +640,8 @@ gda_freetds_provider_get_types (GdaConnection    *cnc,
 {
 	GdaDataModel *model = NULL;
 	_TDSCOLINFO col;
-	GdaValueType gda_type;
-	GdaValue     *value = NULL;
+	GType gda_type;
+	GValue     *value = NULL;
 	gint i = 1;
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
 
@@ -686,25 +657,25 @@ gda_freetds_provider_get_types (GdaConnection    *cnc,
 			/* first fix gda_type */
 			if (row) {
 				value = gda_row_get_value (row, 2);
-				if (gda_value_get_type (value) == GDA_VALUE_TYPE_INTEGER)
-					col.column_size = gda_value_get_integer (value);
+				if (G_VALUE_TYPE (value) == G_TYPE_INT)
+					col.column_size = g_value_get_int (value);
 				else
 					col.column_size = 0;
 				
 				value = gda_row_get_value (row, 3);
-				if (gda_value_get_type (value) != GDA_VALUE_TYPE_TINYINT)
+				if (G_VALUE_TYPE (value) != G_TYPE_CHAR)
 					col.column_type = SYBVARIANT;
 				else 
-					col.column_type = gda_value_get_tinyint (value);
+					col.column_type = g_value_get_char (value);
 				
 				gda_type = gda_freetds_get_value_type (&col);
 				
 				/* col 3: type */
-				gda_value_set_gdatype (value, gda_type);
+				g_value_set_ulong (value, gda_type);
 				
 				/* col 2: comment */
 				value = gda_row_get_value (row, 2);
-				gda_value_set_string (value, "");
+				g_value_set_string (value, "");
 			}
 		}
 	}
@@ -865,19 +836,19 @@ gda_freetds_get_stringresult_of_query (GdaConnection *cnc,
                                        const gint row)
 {
 	GdaDataModel    *model = NULL;
-	GdaValue        *value = NULL;
+	GValue        *value = NULL;
 	gchar           *ret = NULL;
 
 	/* GDA_IS_CONNECTION (cnc) validation in function call */
 	model = gda_freetds_execute_query (cnc, sql);
 	
 	if (model) {
-		value = (GdaValue *) gda_data_model_get_value_at (model,
+		value = (GValue *) gda_data_model_get_value_at (model,
 		                                                  col, row);
 
 		if ((value != NULL)
-		    && (gda_value_get_type (value) != GDA_VALUE_TYPE_NULL)
-		    && (gda_value_get_type (value) != GDA_VALUE_TYPE_UNKNOWN)) {
+		    && (G_VALUE_TYPE (value) != GDA_TYPE_NULL)
+		    && (G_VALUE_TYPE (value) != G_TYPE_INVALID)) {
 			ret = (gchar *) gda_value_stringify (value);
 		}
 		
@@ -920,7 +891,7 @@ gda_freetds_get_fields (GdaConnection *cnc, GdaParameterList *params)
 	parameter = gda_parameter_list_find (params, "name");
 	g_return_val_if_fail (parameter != NULL, NULL);
 
-	table = gda_value_get_string ((GdaValue *) gda_parameter_get_value (parameter));
+	table = g_value_get_string ((GValue *) gda_parameter_get_value (parameter));
 	g_return_val_if_fail (table != NULL, NULL);
 
 	query = g_strdup_printf (TDS_SCHEMA_FIELDS, table);
@@ -949,7 +920,7 @@ static GList* gda_freetds_provider_process_sql_commands(GList         *reclist,
 	g_return_val_if_fail (tds_cnc->tds != NULL, NULL);
 
 /*	arr = gda_freetds_split_commandlist(sql); */
-	arr = g_strsplit (sql, ";", 0);
+	arr = gda_delimiter_split_sql (sql);
 	if (arr) {
 		gint n = 0;
 		while (arr[n]) {
@@ -992,7 +963,7 @@ gda_freetds_provider_class_init (GdaFreeTDSProviderClass *klass)
 	provider_class->get_version = gda_freetds_provider_get_version;
 	provider_class->get_server_version = gda_freetds_provider_get_server_version;
 	provider_class->get_info = NULL;
-	provider_class->supports = gda_freetds_provider_supports;
+	provider_class->supports_feature = gda_freetds_provider_supports;
 	provider_class->get_schema = gda_freetds_provider_get_schema;
 
 	provider_class->get_data_handler = NULL;
@@ -1000,20 +971,14 @@ gda_freetds_provider_class_init (GdaFreeTDSProviderClass *klass)
 	provider_class->get_def_dbms_type = NULL;
 
 	provider_class->open_connection = gda_freetds_provider_open_connection;
-	provider_class->reset_connection = NULL;
 	provider_class->close_connection = gda_freetds_provider_close_connection;
 	provider_class->get_database = gda_freetds_provider_get_database;
 	provider_class->change_database = gda_freetds_provider_change_database;
 
-	provider_class->get_specs = NULL;
-	provider_class->perform_action_params = NULL;
-
-	provider_class->create_database_cnc = gda_freetds_provider_create_database_cnc;
-	provider_class->drop_database_cnc = gda_freetds_provider_drop_database_cnc;
-	provider_class->create_table = NULL;
-	provider_class->drop_table = NULL;
-	provider_class->create_index = NULL;
-	provider_class->drop_index = NULL;
+	provider_class->supports_operation = NULL;
+        provider_class->create_operation = NULL;
+        provider_class->render_operation = NULL;
+        provider_class->perform_operation = NULL;
 
 	provider_class->execute_command = gda_freetds_provider_execute_command;
 	provider_class->get_last_insert_id = NULL;
