@@ -42,19 +42,19 @@ typedef struct _GdaIBMDB2RecordsetPrivate GdaIBMDB2RecordsetPrivate;
 typedef struct _GdaIBMDB2Field            GdaIBMDB2Field;
 
 struct _GdaIBMDB2Recordset {
-	GdaDataModelBase model;
+	GdaDataModelRow model;
 	GdaIBMDB2RecordsetPrivate *priv;
 };
 
 struct _GdaIBMDB2RecordsetClass {
-	GdaDataModelBaseClass parent_class;
+	GdaDataModelRowClass parent_class;
 };
 
 struct _GdaIBMDB2RecordsetPrivate
 {
         GdaConnection *cnc;
 	GdaIBMDB2ConnectionData *conn_data;
-	/* GdaValueType *column_types; */
+	/* GType *column_types; */
 		
 	SQLHANDLE hstmt;
 		
@@ -76,7 +76,7 @@ struct _GdaIBMDB2Field {
         SQLSMALLINT  column_scale;
 	SQLSMALLINT  column_nullable;
         SQLPOINTER   column_data;
-	GdaValueType gda_type;
+	GType gda_type;
 };
 																		
 
