@@ -1,5 +1,5 @@
 /* GDA LDAP provider
- * Copyright (C) 1998-2003 The GNOME Foundation.
+ * Copyright (C) 1998 - 2006 The GNOME Foundation.
  *
  * AUTHORS:
  *      Michael Lausch <michael@lausch.at>
@@ -88,7 +88,7 @@ gda_ldap_provider_class_init (GdaLdapProviderClass *klass)
 	provider_class->get_version = gda_ldap_provider_get_version;
 	provider_class->get_server_version = gda_ldap_provider_get_server_version;
 	provider_class->get_info = NULL;
-	provider_class->supports = gda_ldap_provider_supports;
+	provider_class->supports_feature = gda_ldap_provider_supports;
 	provider_class->get_schema = gda_ldap_provider_get_schema;
 
 	provider_class->get_data_handler = NULL;
@@ -96,20 +96,14 @@ gda_ldap_provider_class_init (GdaLdapProviderClass *klass)
 	provider_class->get_def_dbms_type = NULL;
 
 	provider_class->open_connection = gda_ldap_provider_open_connection;
-	provider_class->reset_connection = NULL;
 	provider_class->close_connection = gda_ldap_provider_close_connection;
 	provider_class->get_database = gda_ldap_provider_get_database;
 	provider_class->change_database = NULL;
 
-	provider_class->get_specs = NULL;
-	provider_class->perform_action_params = NULL;
-
-	provider_class->create_database_cnc = NULL;
-	provider_class->drop_database_cnc = NULL;
-	provider_class->create_table = NULL;
-	provider_class->drop_table = NULL;
-	provider_class->create_index = NULL;
-	provider_class->drop_index = NULL;
+	provider_class->supports_operation = NULL;
+        provider_class->create_operation = NULL;
+        provider_class->render_operation = NULL;
+        provider_class->perform_operation = NULL;
 
 	provider_class->execute_command = NULL;
 	provider_class->get_last_insert_id = NULL;
@@ -307,12 +301,12 @@ gda_ldap_provider_supports (GdaServerProvider *provider,
 static void
 add_string_row (GdaDataModelArray *recset, const gchar *str)
 {
-	GdaValue *value;
+	GValue *value;
 	GList list;
 
 	g_return_if_fail (GDA_IS_DATA_MODEL_ARRAY (recset));
 
-	value = gda_value_new_string (str);
+	g_value_set_string (value = gda_value_new (G_TYPE_STRING), str);
 	list.data = value;
 	list.next = NULL;
 	list.prev = NULL;
@@ -410,13 +404,20 @@ get_ldap_tables (GdaConnection *cnc, GdaParameterList *params)
 				if (oc) {
 					for (j = 0; oc->oc_names[j] != NULL; j++) {
 						GList *value_list = NULL;
+						GValue *tmpval;
 
 						/* fill the recordset */
-						value_list = g_list_append (value_list,
-									    gda_value_new_string (oc->oc_names[j]));
-						value_list = g_list_append (value_list, gda_value_new_string (""));
-						value_list = g_list_append (value_list, gda_value_new_string (""));
-						value_list = g_list_append (value_list, gda_value_new_string (""));
+						g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), oc->oc_names[j]);
+						value_list = g_list_append (value_list, tmpval);
+
+						g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), "");
+						value_list = g_list_append (value_list, tmpval);
+
+						g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), "");
+						value_list = g_list_append (value_list, tmpval);
+
+						g_value_set_string (tmpval = gda_value_new (G_TYPE_STRING), "");
+						value_list = g_list_append (value_list, tmpval);
 
 						gda_data_model_append_values (GDA_DATA_MODEL (recset), value_list, NULL);
 						/*printf ("%s\n", oc->oc_names[j]);*/
